<?php 
	
	// Simple PHP Blog
	// ------------------------------
	// Created by: Alexander Palmo, apalmo <at> bigevilbrain <dot> com
	//
	// RSS/PING/GZIP/EMAIL Code:
	// Contributed by: Javier Gutierrez, guti <at> ya <dot> com
	
	// The Simple PHP Blog is released under the GNU Public License.
	//
	// You are free to use and modify the Simple PHP Blog. All changes 
	// must be uploaded to SourceForge.net under Simple PHP Blog or
	// emailed to apalmo <at> bigevilbrain <dot> com
	//
	// Credit should be give to the original authors and the Simple PHP Blog
	// logo graphic must appear on the site and link to the project
	// on SourceForge.net
	
	// Common functions for the Simple PHP blog.
	
	global $sb_info;
	$sb_info[ 'version' ] = '0.3.7c';
	$sb_info[ 'last_update' ] = '7/06/04';
	
	// error_reporting(E_ALL ^ E_NOTICE);
	error_reporting(0);
	
	global $page_timestamp;
	$page_timestamp = getmicrotime();
	
	require('scripts/sb_display.php');

	// ------------------
	// File I/O Functions
	// ------------------
	function sb_read_file( $filename ) {
		// Safely read a file.
		//
		// Returns either the contents of the file or NULL on fail.
		if ( version_compare( phpversion(), '4.3.0' ) == -1 ) {
			$result = NULL;
			clearstatcache();
			if ( file_exists( $filename ) ) {
				$fp = @fopen( $filename, 'r' );
				if ( $fp ) {
					$lock = flock( $fp, LOCK_SH );
					if ( $lock ) {
						$result = fread( $fp, filesize( $filename )*100 );
						if ( ( strpos( $filename, '.gz' ) !== false ) && ( extension_loaded( 'zlib' ) ) ) {
							$result = gzinflate( substr( $result, 10 ) );
						}
						$lock = flock( $fp, LOCK_UN );
					}
					fclose( $fp );
				}
			}
		}
		else {
			$result=@file_get_contents( $filename );
			if ( ( strpos( $filename, '.gz' ) !== false ) && ( extension_loaded( 'zlib' ) ) ) {
				$result = gzinflate( substr( $result, 10 ) );
			}
		}
		
		return( $result );
	}
	
	function sb_write_file( $filename, $str ) {
		// Safely write a file.
		//
		// Returns true or false.
		
		if ( ( strpos( $filename, '.gz' )!==false ) && ( extension_loaded( 'zlib' ) ) ) {
			$str=gzencode( $str, 9 );
		}
		
		if ( version_compare( phpversion(), '5.0.0' ) == -1 ) {
			$result = false;
			clearstatcache();
			$fp = @fopen( $filename, 'w' );
			if ( $fp ) {
				$lock = flock( $fp, LOCK_EX );
				if ( $lock ) {
					$result = fwrite( $fp, $str );
					$lock = flock( $fp, LOCK_UN );
				}
				fclose( $fp );
			}
		}
		else {
			$result=file_put_contents( $filename, $str );
		}
		
		return( $result );
	}
	
	function sb_folder_listing( $dir, $ext_array ) {
		// Return an array of files in a directory.
		// On fail returns an empty array.
		//
		// Optionally you can pass an array of file
		// extensions to match. Pass an empty array
		// if you want it to return all files.
		//
		$result = array();
		
		clearstatcache();
		if ( is_dir($dir) ) {
			$dhandle = opendir($dir);
			if ( $dhandle ) {
				// Loop through files
				$filename = readdir( $dhandle );
				while ( $filename ) {
					if ( is_file( $dir . $filename ) ) {
						if ( count( $ext_array ) > 0 ) {
							$ok = false;
							for ( $i = 0; $i < count( $ext_array ); $i++ ) {
								if ( strtolower( strrchr( $filename, '.' ) ) == strtolower( $ext_array[$i] ) ) {
									$ok = true;
									break;
								}
							}
						} else {
							$ok = true;
						}
						if ( $ok == true ) {
							array_push( $result, $filename );
						}
					}
					$filename = readdir( $dhandle );
				}
			}
			closedir( $dhandle) ;
		}
		
		sort( $result );
		return( $result );
	}
	
	//Cleaned up
	function sb_delete_file ( $filename ) {
		// Delete a file.
		//
		// I'm guessing that we don't need to lock a file to delete it. Who knows?
		$result = @unlink( $filename );
		
		return( $result );
	}
	
	//Cleaned up
	function sb_delete_directory ( $dirname ) {
		// Delete a directory. Only works if directory is empty.
		//
		// Returns true or false
		$result = @rmdir( $dirname );
		
		return( $result );
	}
	
	function sb_strip_extension ( $filename ) {
		$temp_pos = strpos( $filename, '.' );
		if ( $temp_pos !== false ) {
			$filename = substr( $filename, 0, $temp_pos );
		}
		
		return( $filename );
	}
	
	// -----------------
	// Utility Functions
	// -----------------
	
	function getmicrotime() { 
		list($usec, $sec) = explode(" ", microtime()); 
		return ((float)$usec + (float)$sec); 
	}

	function clean_post_text( $str ) {
		// Cleans post text input.
		//
		// Strip out pipes and HTML-ize entities. Use charset from the 
		// language file to make sure we're only encoding stuff that
		// needs to be encoded.
		global $lang_string;
		$str = str_replace( '|', ':', $str );
		$str = htmlspecialchars( $str, ENT_QUOTES, $lang_string['php_charset'] );

		return ( $str );
	}
	
	function blog_to_html( $str, $comment_mode, $strip_all_tags ) {
		// Convert Simple Blog tags to HTML.
		//
		// Search and replace simple tags. These tags don't have any
		// special attributes so we can do a str_replace() on them.
		//
		// 
		//
		// (Could use str_ireplace() but it's only supported in PHP 5.)
		global $blog_config;
		
		if ( $comment_mode ) {
			$tag_arr = array();
			if ( in_array( 'i', $blog_config[ 'comment_tags_allowed' ] ) ) { array_push( $tag_arr, 'i' ); }
			if ( in_array( 'b', $blog_config[ 'comment_tags_allowed' ] ) ) { array_push( $tag_arr, 'b' ); }
			if ( in_array( 'blockquote', $blog_config[ 'comment_tags_allowed' ] ) ) { array_push( $tag_arr, 'blockquote' ); }
			if ( in_array( 'strong', $blog_config[ 'comment_tags_allowed' ] ) ) { array_push( $tag_arr, 'strong' ); }
			if ( in_array( 'em', $blog_config[ 'comment_tags_allowed' ] ) ) { array_push( $tag_arr, 'em' ); }
			if ( in_array( 'hN', $blog_config[ 'comment_tags_allowed' ] ) ) { array_push( $tag_arr, 'h1', 'h2', 'h3', 'h4', 'h5', 'h6' ); }
			if ( in_array( 'del', $blog_config[ 'comment_tags_allowed' ] ) ) { array_push( $tag_arr, 'del' ); }
			if ( in_array( 'ins', $blog_config[ 'comment_tags_allowed' ] ) ) { array_push( $tag_arr, 'ins' ); }
			if ( in_array( 'strike', $blog_config[ 'comment_tags_allowed' ] ) ) { array_push( $tag_arr, 'strike' ); }
			if ( in_array( 'pre', $blog_config[ 'comment_tags_allowed' ] ) ) { array_push( $tag_arr, 'pre' ); }
			if ( in_array( 'code', $blog_config[ 'comment_tags_allowed' ] ) ) { array_push( $tag_arr, 'code' ); }
		} else {
			$tag_arr = array('i', 'b', 'blockquote', 'strong', 'em', 'h1', 'h2', 'h3', 'h4', 'h5', 'h6', 'del', 'ins', 'strike', 'pre', 'code' );
		}
		
		// Build search and replace arrays.
		$search_arr = array();
		$replace_arr = array();
		for ( $i = 0; $i < count( $tag_arr ); $i++ ) {
			$tag = $tag_arr[$i];
			array_push( $search_arr,  '[' . strtolower( $tag ) . ']',  '[' . strtoupper( $tag ) . ']' );
			if ( $strip_all_tags ) {
				array_push( $replace_arr, '',  '' );
			} else {
				array_push( $replace_arr, '<' . strtolower( $tag ) . '>',  '<' . strtoupper( $tag ) . '>' );
			}
			array_push( $search_arr,  '[/' . strtolower( $tag ) . ']', '[/' . strtoupper( $tag ) . ']' );
			if ( $strip_all_tags ) {
				array_push( $replace_arr, '',  '' );
			} else {
				array_push( $replace_arr, '</' . strtolower( $tag ) . '>', '</' . strtoupper( $tag ) . '>' );
			}
		}
		
		// Do str_replace() replacement.
		$str = str_replace( $search_arr, $replace_arr, $str);
		
		// Replace [url] Tags:
		// The [url] tag has an optional "new" attribute. The "new"
		// attribute determines whether to open the link in the
		// same window or a new window.
		// new   - (true/false)
		//
		// [url=http://xxx]xxx[/url] 
		// [url=http://xxx new=true]xxx[/url]
		if ( $comment_mode ) {
			if ( in_array( 'url', $blog_config[ 'comment_tags_allowed' ] ) && $strip_all_tags === false ) {
				$str = replace_url_tag( $str, '[url=', ']', '[/url]', false );
				$str = replace_url_tag( $str, '[URL=', ']', '[/URL]', false );
			} else {
				$str = replace_url_tag( $str, '[url=', ']', '[/url]', true );
				$str = replace_url_tag( $str, '[URL=', ']', '[/URL]', true );
			}
		} else {
			if ( $strip_all_tags ) {
				$str = replace_url_tag( $str, '[url=', ']', '[/url]', true );
				$str = replace_url_tag( $str, '[URL=', ']', '[/URL]', true );
			} else {
				$str = replace_url_tag( $str, '[url=', ']', '[/url]', false );
				$str = replace_url_tag( $str, '[URL=', ']', '[/URL]', false );
			}
		}
		
		// Replace [img] Tags:
		// The [img] tag has an number of optional attributes -
		// width  - width of image in pixels
		// height - height of image in pixels
		// popup  - (true/false)
		// float  - (left/right)
		//
		// [img=http://xxx]
		// [img=http://xxx width=xxx height=xxx popup=true float=left]
		if ( $comment_mode ) {
			if ( in_array( 'img', $blog_config[ 'comment_tags_allowed' ] ) && $strip_all_tags === false  ) {
				$str = replace_img_tag( $str, '[img=', ']', false );
				$str = replace_img_tag( $str, '[IMG=', ']', false );
			} else {
				$str = replace_img_tag( $str, '[img=', ']', true );
				$str = replace_img_tag( $str, '[IMG=', ']', true );
			}
		} else {
			if ( $strip_all_tags ) {
				$str = replace_img_tag( $str, '[img=', ']', true );
				$str = replace_img_tag( $str, '[IMG=', ']', true );
			} else {
				$str = replace_img_tag( $str, '[img=', ']', false );
				$str = replace_img_tag( $str, '[IMG=', ']', false );
			}
		}
		
		// Emoticons
		// $smile_arr = array()
		// array_push( $smile_arr, array( ':)',  'smile.png') );
		// array_push( $smile_arr, array( ':-)', 'smile.png') );
		// array_push( $smile_arr, array( ':(',  'frown.png') );
		// array_push( $smile_arr, array( ':-(', 'frown.png') );
		// array_push( $smile_arr, array( ':o',  'surprised.png') );
		// array_push( $smile_arr, array( ':-o', 'surprised.png') );
		// array_push( $smile_arr, array( ':O',  'surprised.png') );
		// array_push( $smile_arr, array( ':-O', 'surprised.png') );
		// array_push( $smile_arr, array( ':p',  'sticking_out_tongue.png') );
		// array_push( $smile_arr, array( ':-p', 'sticking_out_tongue.png') );
		// array_push( $smile_arr, array( ':P',  'sticking_out_tongue.png') );
		// array_push( $smile_arr, array( ':-P', 'sticking_out_tongue.png') );
		// array_push( $smile_arr, array( ':D',  'laughing.png') );
		// array_push( $smile_arr, array( ':-D', 'laughing.png') );
		// for ( $i = 0; $i < count( $smile_arr ); $i++ ) {
		// 	$str = str_replace( $smile_arr[$i][0], '<img src="interface/emoticons/'.$smile_arr[$i][1].'" alt="$smile_arr[$i][0]">', $str );
		// }
		
		// Selectively replace line breaks and/or decode html entities.
		if ( $comment_mode ) {		
			if ( in_array( 'html', $blog_config[ 'comment_tags_allowed' ] ) && $strip_all_tags === false ) {
				$str = replace_html_tag( $str, false );
			} else {
				$str = replace_html_tag( $str, true );
			}
		} else {
			if ( $strip_all_tags ) {
				$str = replace_html_tag( $str, true );
			} else {
				$str = replace_html_tag( $str, false );
			}
		}
		
		return ( $str );
	}
	
	function replace_html_tag( $str, $strip_tags ) {
		// Replacements for HTML tags. Sub-function of blog_to_html.
		//
		// This function decodes HTML entities that are located between
		// HTML tags. Also, inserts <br />'s for new lines only if blocks
		// are outside the HTML tags.
		global $lang_string;
		
		$str_out = NULL;
		$tag_begin = '[html]';
		$tag_end = '[/html]';
		
		// Search for the openning HTML tag. Tag could be either upper or
		// lower case so we want to find the nearest one.
		//
		// Get initial $str_offset value.
		$temp_lower = strpos( $str, strtolower( $tag_begin ) );
		$temp_upper = strpos( $str, strtoupper( $tag_begin ) );
		if ( $temp_lower === false ) {
			if ( $temp_upper === false ) {
				$str_offset = false;
			} else {
				$str_offset = $temp_upper;
			}
		} else {
			if ( $temp_upper === false ) {
				$str_offset = $temp_lower;
			} else {
				$str_offset = min( $temp_upper, $temp_lower );
			}
		}
		
		while ( $str_offset !== false ) {
			// Store all the text BEFORE the openning HTML tag.
			//
			// Also, replace hard returns with '<br />' tags.
			$temp_str = substr( $str, 0, $str_offset );
			$temp_str = str_replace( chr(10), '<br />', $temp_str );
			$str_out = $str_out . $temp_str;
			
			// Store all text AFTER the tag
			$str = substr( $str, $str_offset + strlen( $tag_begin ) );
		
			// Search for the closing HTML tag. Find the nearest one.
			$temp_lower = strpos( $str, strtolower( $tag_end ) );
			$temp_upper = strpos( $str, strtoupper( $tag_end ) );
			if ( $temp_lower === false ) {
				if ( $temp_upper === false ) {
					$str_offset = false;
				} else {
					$str_offset = $temp_upper;
				}
			} else {
				if ( $temp_upper === false ) {
					$str_offset = $temp_lower;
				} else {
					$str_offset = min( $temp_upper, $temp_lower );
				}
			}
			
			if ( $str_offset !== false ) {
				// Store all the text BETWEEN the HTML tags.
				//
				// Also, decode HTML entities between the tags.
				$temp_str = substr( $str, 0, $str_offset );
				if ( $strip_tags === false ) {
					$temp_str = html_entity_decode( $temp_str, ENT_QUOTES, $lang_string['php_charset'] );
				}
				$str_out = $str_out . $temp_str;
				
				// Store sub_string after the tag.
				$str = substr( $str, $str_offset + strlen( $tag_end ) );
				
				// Search for openning HTML tag again.
				$temp_lower = strpos( $str, strtolower( $tag_begin ) );
				$temp_upper = strpos( $str, strtoupper( $tag_begin ) );
				if ( $temp_lower === false ) {
					if ( $temp_upper === false ) {
						$str_offset = false;
					} else {
						$str_offset = $temp_upper;
					}
				} else {
					if ( $temp_upper === false ) {
						$str_offset = $temp_lower;
					} else {
						$str_offset = min( $temp_upper, $temp_lower );
					}
				}
			}
		}
		
		// Append remainder of text.
		//
		// All this text will be outside of any HTML tags so
		// we need to encode the line breaks.
		$str = str_replace( chr(10), '<br />', $str );
		$str = $str_out . $str;
		
		return ( $str );
	
	}
	
	function replace_url_tag( $str, $tag_begin, $tag_end, $tag_close, $strip_tags ) {
		// Replacements for URL tags. Sub-function of blog_to_html.
		//
		// If $strip_tags == true then it will strip out the tag
		// instead of making them HTML.
		$str_out = NULL;
		
		// Search for the beginning part of the tag.
		$str_offset = strpos( $str, $tag_begin );
		while ( $str_offset !== false ) {
			// Store sub_string before the tag.
			$str_out = $str_out . substr( $str, 0, $str_offset );
			// Store sub_string after the tag.
			$str = substr( $str, $str_offset + strlen( $tag_begin ) );
			
			// Search for the ending part of the tag.
			$str_offset = strpos( $str, $tag_end );
			if ( $str_offset !== false ) {
				
				if ( $strip_tags == false ) {
					// Store attribues BETWEEN between the tags.
					$attrib_array = explode( ' ', substr( $str, 0, $str_offset ) );
					$attrib_new = NULL;
					
					if ( is_array( $attrib_array ) ) {
						$str_url = $attrib_array[0];
						
						for ( $i = 1; $i < count( $attrib_array ); $i++ ) {
							$temp_arr = explode( '=', $attrib_array[$i] );
							if ( is_array( $temp_arr ) && count( $temp_arr ) == 2 ) {
								switch ( $temp_arr[0] ) {
									case 'new';
										$attrib_new = $temp_arr[1];
										break;
								}
							}
						}
					} else {
						$str_url = $attrib_array;
					}
					
					// Append HTML tag.
					if ( isset( $attrib_new ) ) {
						if ( $attrib_new == 'false' ) {
							$str_out = $str_out . '<a href="' . $str_url . '">';						
						} else {
							$str_out = $str_out . '<a href="' . $str_url . '" target="_blank">';
						}
					} else {
						$str_out = $str_out . '<a href="' . $str_url . '" target="_blank">';					
					}
				}
				
				// Store sub_string AFTER the tag.
				$str = substr( $str, $str_offset + strlen( $tag_end ) );
				
				// Look for closing tag.
				$str_offset = strpos( $str, $tag_close );
				if ( $str_offset !== false ) {
					$str_link = substr( $str, 0, $str_offset );
					if ( $strip_tags == false ) {
						$str_out = $str_out . $str_link . '</a>';
					} else {
						$str_out = $str_out . $str_link;
					}					
					$str = substr( $str, $str_offset + strlen( $tag_close ) );
				}
				
				// Search for next beginning tag.
				$str_offset = strpos( $str, $tag_begin );
			}
		}
		
		// Append remainder of tag.
		$str = $str_out . $str;
		
		return ( $str );
	}
	
	function replace_img_tag( $str, $tag_begin, $tag_end, $strip_tags ) {
		// Replacements for IMG tags. Sub-function of blog_to_html.
		// 
		// I made this another function because I wanted to be able
		// to call it for upper and lower case '[img=]' tags...
		//
		// If $strip_tags == true then it will strip out the tag
		// instead of making them HTML.
		global $theme_vars;
		
		$str_out = NULL;
		
		// Search for the beginning part of the tag.
		$str_offset = strpos( $str, $tag_begin );
		while ( $str_offset !== false ) {
			// Store sub_string before the tag.
			$str_out = $str_out . substr( $str, 0, $str_offset );
			// Store sub_string after the tag.
			$str = substr( $str, $str_offset + strlen( $tag_begin ) );
			
			// Search for the ending part of the tag.
			$str_offset = strpos( $str, $tag_end );
			if ( $str_offset !== false ) {
			
				if ( $strip_tags == true ) {
					
					// Store sub_string after the tag.
					$str = substr( $str, $str_offset + strlen( $tag_end ) );
					// Search for next beginning tag.
					$str_offset = strpos( $str, $tag_begin );
					
				} else {
				
					// Store attribues between between the tags.
					$attrib_array = explode( ' ', substr( $str, 0, $str_offset ) );
					$attrib_width = NULL;
					$attrib_height = NULL;
					$attrib_popup = NULL;
					$attrib_float = NULL;
					
					if ( is_array( $attrib_array ) ) {
						$str_url = $attrib_array[0];
						
						for ( $i = 1; $i < count( $attrib_array ); $i++ ) {
							$temp_arr = explode( '=', $attrib_array[$i] );
							if ( is_array( $temp_arr ) && count( $temp_arr ) == 2 ) {
								switch ( $temp_arr[0] ) {
									case 'width';
										$attrib_width = intval( $temp_arr[1] );
										break;
									case 'height';
										$attrib_height = intval( $temp_arr[1] );
										break;
									case 'popup';
										$attrib_popup = $temp_arr[1];
										break;
									case 'float';
										$attrib_float = $temp_arr[1];
										break;
								}
							}
						}
					} else {
						$str_url = $attrib_array;
					}
				
					// Grab image size and calculate scaled sizes
					
					// if ( file_exists( $str_url ) !== false ) {
					$img_size = @getimagesize( $str_url );
					if ( $img_size !== false ) {
						$width = $img_size[0];
						$height = $img_size[1];
						$max_image_width = $theme_vars['max_image_width'];
						
						if ( isset( $attrib_width ) && isset( $attrib_height ) ) {
								$width = $attrib_width;
								$height = $attrib_height;
						} else {
							if ( isset( $attrib_width ) ) {
								$height = round( $height * ( $attrib_width / $width ) );
								$width = $attrib_width;
							}
							
							if ( isset( $attrib_height ) ) {
								$width = round( $width * ( $attrib_height / $height ) );
								$height = $attrib_height;
							}
						}
						
						if ( $width > $max_image_width ) {
							$height = round( $height * ( $max_image_width / $width ) );
							$width = $max_image_width;
						}
						
						
						if ( isset( $attrib_popup ) ) {
							if ( $attrib_popup == 'true' ) {
								$str_out = $str_out . '<a href="javascript:openpopup(\'' . $str_url . '\','.$img_size[0].','.$img_size[1].',false);"><img src="' . $str_url . '" width='.$width.' height='.$height.' border=0 alt=\'\'';
								if ( isset( $attrib_float ) ) {
									switch ( $attrib_float ) {
										case 'left';
											$str_out = $str_out . ' id="img_float_left"';
											break;
										case 'right';
											$str_out = $str_out . ' id="img_float_right"';
											break;
									}
								}
								$str_out = $str_out . '></a>';
							} else {
								$str_out = $str_out . '<img src="' . $str_url . '" width='.$width.' height='.$height.' border=0 alt=\'\'';
								if ( isset( $attrib_float ) ) {
									switch ( $attrib_float ) {
										case 'left';
											$str_out = $str_out . ' id="img_float_left"';
											break;
										case 'right';
											$str_out = $str_out . ' id="img_float_right"';
											break;
									}
								}
								$str_out = $str_out . '>';
							}
						} else {
							if ( $width != $img_size[0] || $height != $img_size[1] ) {
								$str_out = $str_out . '<a href="javascript:openpopup(\'' . $str_url . '\','.$img_size[0].','.$img_size[1].',false);"><img src="' . $str_url . '" width='.$width.' height='.$height.' border=0 alt=\'\'></a>';							
							} else {
								$str_out = $str_out . '<img src="' . $str_url . '" width='.$width.' height='.$height.' border=0 alt=\'\'>';
							}
						}
										
						// Store sub_string after the tag.
						$str = substr( $str, $str_offset + strlen( $tag_end ) );
						// Search for next beginning tag.
						$str_offset = strpos( $str, $tag_begin );
					} else {
						// Append HTML tag.
						if ( isset( $attrib_popup ) ) {
							if ( $attrib_popup == 'true' ) {
								$str_out = $str_out . '<a href="javascript:openpopup(\'' . $str_url . '\',800,600,false);"><img src="' . $str_url . '" border=0 alt=\'\'></a>';						
							} else {
								$str_out = $str_out . '<img src="' . $str_url . '" border=0 alt=\'\'>';		
							}
						} else {
							$str_out = $str_out . '<a href="javascript:openpopup(\'' . $str_url . '\',800,600,false);"><img src="' . $str_url . '" border=0 alt=\'\'></a>';	
						}
										
						// Store sub_string after the tag.
						$str = substr( $str, $str_offset + strlen( $tag_end ) );
						// Search for next beginning tag.
						$str_offset = strpos( $str, $tag_begin );
					}
				}
			}
		}
		
		// Append remainder of tag.
		$str = $str_out .  $str;
		
		return ( $str );
	}
	
	function safe_version_compare( $versionA, $versionB ) {
		// This is a PHP < 4.1 safe version compare function.
		// The version_compare function was introduced in 4.1
		// so there's no way to compare older versions.
		//
		// returns -1 if the first version is lower than the second,
		// 0 if they are equal, and +1 if the second is lower.
		//
		// return NULL on error.
		
		$arrayA = explode( '.', $versionA );
		$arrayB = explode( '.', $versionB );
		
		$count = min( count( $arrayA ), count( $arrayB ) );
		
		$result = NULL;
		if ( $count > 0) {		
			for ( $i = 0; $i < $count; $i++ ) {
				$intA = intval( $arrayA[$i] );
				$intB = intval( $arrayB[$i] );
				if ( $intA == $intB ) {
					$result = 0;
				} else if ( $intA > $intB ) {
					$result = 1;
				} else if ( $intA < $intB ) {
					$result = -1;
				}
				if ( $result != 0 ) {
					break;
				}
			}
		}
		
		return ( $result );
	}
	
	// -------------------------
	// Session & Login Functions
	// -------------------------
	
	function logged_in ( $redirect_to_login, $redirect_to_setup ) {
		// Check if the user is logged in and has a session.
		//
		// Session data is stored in a client-side cookie.
		// Login flag is stored in the session cookie.

		// Support for PHP >= 4.1.0
		// Check if the user has a client-side cookie.
		if ( isset( $_COOKIE['my_id'] ) ) {
			$my_id = $_COOKIE['my_id'];
			
			// Turn off URL SIDs.
			// ini_set( 'session.use_trans_sid', '0' );
			
			// Init the session.
			session_id($my_id);
			session_start ();
			
			// Check if they're logged in.
			if ( isset( $_SESSION['logged_in'] ) && $_SESSION['logged_in'] == 'yes' ) {
				if ( $_SESSION['site_path'] === dirname($_SERVER['PHP_SELF']) ) {
					return ( true );
				}
			}
		}
		
		// Special Case:
		// If there's no password file then need to redirect them.
		$passFile = 'config/password.txt';
		if ( !file_exists( $passFile ) ) {
			if ( $redirect_to_setup ) {
				$relative_url = 'install00.php';
				if ( dirname($_SERVER['PHP_SELF']) == '\\' ) {
					// Hosted at root.
					header('Location: http://'.$_SERVER['HTTP_HOST'].'/'.$relative_url);
				} else {
					// Hosted in sub-directory.
					header('Location: http://'.$_SERVER['HTTP_HOST'].dirname($_SERVER['PHP_SELF']).'/'.$relative_url);
				}
				
				exit;
			} else {
				return ( -1 );
			}
		}
		
		if ( $redirect_to_login ) {
			$relative_url = 'index.php';
			if ( dirname($_SERVER['PHP_SELF']) == '\\' ) {
				// Hosted at root.
				header('Location: http://'.$_SERVER['HTTP_HOST'].'/'.$relative_url);
			} else {
				// Hosted in sub-directory.
				header('Location: http://'.$_SERVER['HTTP_HOST'].dirname($_SERVER['PHP_SELF']).'/'.$relative_url);
			}
			
			exit;
		} else {
			return ( false );
		}
	}
	
	function logout () {
		// Log Out
		//
		// Support for PHP >= 4.1.0
		if ( isset( $_COOKIE['my_id'] ) ) {
			$my_id = $_COOKIE['my_id'];
			
			// Turn off URL SIDs.
			// ini_set( 'session.use_trans_sid', '0' );
			
			// Init the session.
			session_id($my_id);
			session_start();
			session_unset($_SESSION['logged_in']);
			session_unset($_SESSION['site_path']);
			session_destroy();
		}
	}
	
	function create_password ( $user, $pass ) {
		// Generate and store password hash
		//
		$mypasswd = $user.$pass;
		$hashed = crypt($mypasswd); 
		
		// Save File
		$filename = 'config/password.txt';
		$result = sb_write_file( $filename, $hashed );
		
		if ( $result ) {
			// Start Session and Set Cookie
			// ini_set( 'session.use_trans_sid', '0' );
			@session_start();
			
			// Support for PHP >= 4.1.0
			$_SESSION['logged_in'] = 'yes';
			$_SESSION['site_path'] = dirname($_SERVER['PHP_SELF']);
			setcookie('my_id',session_id());
			
			return ( true );
		} else {
			// Error:
			// Probably couldn't create file...
			return ( $filename );
		}
	}
	
	function check_password ( $user, $pass ) {
		// Check password against hashed password file
		//
		$passFile = 'config/password.txt';
		$hashed = sb_read_file( $passFile );
		
		// Verify Password
		$mypasswd = $user.$pass;
		$chkpass = crypt($mypasswd, $hashed); 
		if ($hashed === $chkpass) { 
			// Start Session and Set Cookie
			// ini_set( 'session.use_trans_sid', '0' );
			@session_start();
			
			// Support for PHP >= 4.1.0
			$_SESSION['logged_in'] = 'yes';
			$_SESSION['site_path'] = dirname($_SERVER['PHP_SELF']);
			
			setcookie('my_id',session_id());
			
			return ( true );
		} else { 
			return ( false );
		}
	}
	
	// ----------------
	// Config Functions
	// ----------------
	
	function read_config ( ) {
		// Read config information from file.
		//
		global $blog_config, $lang_string;
		
		$blog_config = array();
		
		// LOAD CONFIG INFORMATION
		$contents = sb_read_file( 'config/config.txt' );
		if ( $contents ) {
			$temp_configs = explode('|', $contents);
			$config_keys = array( 	'blog_title', 
									'blog_author',
									'blog_footer',
									'blog_language',
									'blog_entry_order',
									'blog_comment_order',
									'blog_enable_comments',
									'blog_max_entries',
									'blog_comments_popup',
									'comment_tags_allowed',
									'blog_email',
									'blog_enable_gzip_txt',
									'blog_enable_gzip_output',
									'blog_email_notification',
									'blog_send_pings',
									'blog_ping_urls' );
									
			for ( $i = 0; $i < count( $temp_configs ); $i++ ) {
				$key = $config_keys[ $i ];
				$blog_config[ $key ] = $temp_configs[ $i ];
			}
		}
		
		// I've got a minor issue here. You have to set the language before you
		// can call strings.php. The strings file doesn't get called until after
		// read_config. So... I'm just putting in some english values here....
		
		if ( !isset( $blog_config[ 'blog_language' ] ) ) {
			$blog_config[ 'blog_language' ] = 'english';
		}
		
		if ( !isset( $blog_config[ 'blog_title' ] ) ) {
			$blog_config[ 'blog_title' ]= 'No Title'; // $lang_string['sb_default_title'];
		}
		
		if ( !isset( $blog_config[ 'blog_author' ] ) ) {
			$blog_config[ 'blog_author' ] = 'No Author'; // $lang_string['sb_default_author'];
		}
		
		if ( !isset( $blog_config[ 'blog_email' ] ) ) {
			$blog_config[ 'blog_email' ] = 'email@myblog.com';
		}
		
		if ( !isset( $blog_config[ 'blog_footer' ] ) ) {
			$blog_config[ 'blog_footer' ] = 'No Footer'; // $lang_string['sb_default_footer'];
		}
		
		if ( !isset( $blog_config[ 'blog_entry_order' ] ) ) {
			$blog_config[ 'blog_entry_order' ] = 'new_to_old';
		}
		
		if ( !isset( $blog_config[ 'blog_comment_order' ] ) ) {
			$blog_config[ 'blog_comment_order' ] = 'new_to_old';
		}
		
		if ( !isset( $blog_config[ 'blog_enable_comments' ] ) ) {
			$blog_config[ 'blog_enable_comments' ] = 1;
		}
		
		if ( !isset( $blog_config[ 'blog_max_entries' ] ) ) {
			$blog_config[ 'blog_max_entries' ] = 5;
		}
		
		if ( !isset( $blog_config[ 'blog_comments_popup' ] ) ) {
			$blog_config[ 'blog_comments_popup' ] = 1;
		}
		
		if ( !isset( $blog_config[ 'comment_tags_allowed' ] ) ) {
			$blog_config[ 'comment_tags_allowed' ] = explode(',', 'b,i,strong,em,url');
		} else {
			$blog_config[ 'comment_tags_allowed' ] = explode(',', $blog_config[ 'comment_tags_allowed' ]);
		}
		
		if ( !isset( $blog_config[ 'blog_enable_gzip_txt' ] ) ) {
			$blog_config[ 'blog_enable_gzip_txt' ] = 0;
		}
		
		if ( !isset( $blog_config[ 'blog_enable_gzip_output' ] ) ) {
			$blog_config[ 'blog_enable_gzip_output' ] = 0;
		}
		
		if ( !isset( $blog_config[ 'blog_email_notification' ] ) ) {
			$blog_config[ 'blog_email_notification' ] = 0;
		}
		
		if ( !isset( $blog_config[ 'blog_send_pings' ] ) ) {
			$blog_config[ 'blog_send_pings' ] = 0;
		}
		
		if ( !isset( $blog_config[ 'blog_ping_urls' ] ) ) {
			$blog_config[ 'blog_ping_urls' ] = '';
		}
		
		// READ META-DATA INFORMATION
		$contents = sb_read_file( 'config/metainfo.txt' );
		if ( $contents ) {
			$temp_configs = explode('|', $contents);
			$config_keys = array( 	'info_keywords', 
									'info_description',
									'info_copyright', );
									
			for ( $i = 0; $i < count( $temp_configs ); $i++ ) {
				$key = $config_keys[ $i ];
				$blog_config[ $key ] = $temp_configs[ $i ];
			}
		}
		
		if ( !isset( $blog_config[ 'info_keywords' ] ) ) {
			$blog_config[ 'info_keywords' ] = '';
		}
		
		if ( !isset( $blog_config[ 'info_description' ] ) ) {
			$blog_config[ 'info_description' ]= '';
		}
		
		if ( !isset( $blog_config[ 'info_copyright' ] ) ) {
			$blog_config[ 'info_copyright' ] = '';
		}
		
		// LOAD THEME
		read_theme();
		
		global $blog_theme;
		require('themes/' . $blog_theme . '/themes.php');
		
		// LOAD COLORS
		read_colors();
		
		// Start GZIP Output
		if ( $blog_config[ 'blog_enable_gzip_output' ] ) {
			sb_gzoutput ();
		}
	}
	
	function write_config ( $blog_title, $blog_author, $blog_email, $blog_footer, $blog_language, $blog_entry_order, $blog_comment_order, $blog_enable_comments, $blog_max_entries, $blog_comments_popup, $comment_tags_allowed, $blog_enable_gzip_txt, $blog_enable_gzip_output, $blog_email_notification, $blog_send_pings, $blog_ping_urls ) {
		// Save config information to file.
		//
		$array = array( clean_post_text( $blog_title ),
						clean_post_text( $blog_author ),
						clean_post_text( $blog_footer ),
						$blog_language,
						$blog_entry_order,
						$blog_comment_order,
						$blog_enable_comments,
						$blog_max_entries,
						$blog_comments_popup,
						$comment_tags_allowed,
						clean_post_text( $blog_email ),
						$blog_enable_gzip_txt,
						$blog_enable_gzip_output,
						$blog_email_notification,
						$blog_send_pings,
						clean_post_text( $blog_ping_urls ) );
						
		$str = implode('|', $array);
		
		if (!file_exists('config')) {
			$oldumask = umask(0);
			$ok = mkdir( 'config', 0777 );
			umask( $oldumask );
		}
		
		$filename = 'config/config.txt';
		$result = sb_write_file( $filename, $str );
		
		if ( $result ) {
			return ( true );
		} else {
			// Error:
			// Probably couldn't create file...
			return ( $filename );
		}
	}
	
	function write_metainfo ( $info_keywords, $info_description, $info_copyright ) {
		// Save information to file.
		//
		$array = array( clean_post_text( $info_keywords ),
						clean_post_text( $info_description ),
						clean_post_text( $info_copyright ) );
						
		$str = implode('|', $array);
		
		if (!file_exists('config')) {
			$oldumask = umask(0);
			$ok = mkdir( 'config', 0777 );
			umask( $oldumask );
		}
		
		$filename = 'config/metainfo.txt';
		$result = sb_write_file( $filename, $str );
		
		if ( $result ) {
			return ( true );
		} else {
			// Error:
			// Probably couldn't create file...
			return ( $filename );
		}
	}
	
	function write_theme ( $blog_theme ) {
		// Save theme information to file.
		//
		$array = array( clean_post_text( $blog_theme ) );
						
		$str = implode('|', $array);
		
		if (!file_exists('config')) {
			$oldumask = umask(0);
			$ok = mkdir('config', 0777 );
			umask($oldumask);
		}
		
		$filename = 'config/theme.txt';
		$result = sb_write_file( $filename, $str );
		
		if ( $result ) {
			return ( true );
		} else {
			// Error:
			// Probably couldn't create file...
			return ( $filename );
		}
	}
	
	function read_theme ( ) {
		// Read theme information from file.
		//
		global $blog_theme;
		
		$blog_theme = 'default';
		
		$filename = 'config/theme.txt';
		$result = sb_read_file( $filename );
		
		if ( $result ) {
			list( $blog_theme ) = explode('|', $result);		
		} else {
			// Error:
			// Probably couldn't read file.
			return ( $filename );
		}
	}
	
	// ----------------------
	// Color Config Functions
	// ----------------------
	
	function read_colors ( ) {
		// Read color information from file.
		//
		global $user_colors, $blog_theme;
		$color_def = theme_default_colors();
		for ( $i = 0; $i < count( $color_def ); $i++ ) {
			$user_colors[ $color_def[$i]['id'] ] = $color_def[$i]['default'];
		}
		
		$filename = 'config/colors-' . $blog_theme . '.txt';
		$result = sb_read_file( $filename );
		if ( $result ) {
			$saved_colors = explode('|', $result);
			for ( $i = 0; $i < count( $saved_colors ); $i = $i + 2 ) {
				$id = $saved_colors[$i];
				$color = $saved_colors[$i+1];
				$user_colors[ $id ] = $color;
			}
		}
	}
	 
	function write_colors ( $post_array ) {
		// Save color information to file.
		//
		global $blog_theme;
			
		$str = implode('|', $post_array);
		
		$filename = 'config/colors-' . $blog_theme . '.txt';
		$result = sb_write_file( $filename, $str );
		
		if ( $result ) {
			return ( true );
		} else {
			// Error:
			// Probably couldn't create file...
			return ( $filename );
		}
	}
	
	// --------------------
	// Blog Entry Functions
	// --------------------
	
	function write_entry ( $blog_subject, $blog_text, $updateFile ) {
		// Save new entry or update old entry
		//
		// $updateFile will either be NULL or the name of the file
		// which is being updated (i.e. entry040603-140634)
		global $blog_config;
		
		clearstatcache();
		
		if ( $updateFile == true ) {
			// Updating an entry
			//
			// We need to grab the date of the old entry.
			if ( file_exists( $updateFile . '.txt' ) ) {
				$oldEntryFile = $updateFile . '.txt';
			} elseif ( file_exists( $updateFile . '.txt.gz' ) ) {
				$oldEntryFile = $updateFile . '.txt.gz';
			}
			$str = sb_read_file( $oldEntryFile );
			list( $temp_subject, $blog_date, $temp_text ) = explode( '|', $str );
			
			if ( $blog_config[ 'blog_enable_gzip_txt' ] ) {
				if ( file_exists( $updateFile . '.txt' ) ) {
					sb_delete_file( $updateFile . '.txt' );
				}
				$entryFile = $updateFile . '.txt.gz';
			} else {
				if ( file_exists( $updateFile . '.txt.gz' ) ) {
					sb_delete_file( $updateFile . '.txt.gz' );
				}
				$entryFile = $updateFile . '.txt';
			}
			
			// (We're just going to assume that the files and dirs exist...)
		} else {
			// Create an entry
			//
			// This is going to be a new entry.
			// We might need to create some directories.
			//
			// The directory and file structure is:
			// 'content/YY/MM/entryYYMMDD-HHMMSS.txt'
			// 'F j, Y, g:i a'
			$blog_date = time();
			
			if (!file_exists('content')) {
				$oldumask = umask(0);
				$ok = mkdir('content', 0777 );
				umask($oldumask);
			}
			
			$dir = 'content/';
			$y = date('y');
			if (!file_exists($dir.$y)) {
				$oldumask = umask(0);
				$ok = mkdir($dir.$y, 0777 );
				umask($oldumask);
				if (!$ok) {
					return ( 'Couldn\'t creating directory: '.$dir.$y );
				}
			}
			$m = date('m');
			if (!file_exists($dir.$y.'/'.$m)) {
				$oldumask = umask(0);
				$ok = mkdir($dir.$y.'/'.$m, 0777 );
				umask($oldumask);
				if (!$ok) {
					return ( 'Couldn\'t creating directory: '.$dir.$y.'/'.$m );
				}
			}
			
			$stamp = date('ymd-His');
			if ( $blog_config[ 'blog_enable_gzip_txt' ] ) {
				$entryFile = $dir.$y.'/'.$m.'/'.'entry'.$stamp.'.txt.gz';
			} else {
				$entryFile = $dir.$y.'/'.$m.'/'.'entry'.$stamp.'.txt';
			}
		}
		
		// Save the file
		$array = array( clean_post_text( $blog_subject ), $blog_date, clean_post_text( $blog_text ) );
		$str = implode('|', $array);
		
		$result = sb_write_file( $entryFile, $str );
		
		if ( $result ) {
			// Send Pings
			$url_array = explode( ',', $blog_config[ 'blog_ping_urls' ] );
			if ( is_array( $url_array ) ) {
				for ( $i = 0; $i < count( $url_array ); $i++ ) {
					sb_ping( $url_array[ $i ] );
				}
			}
			
			return ( true );
		} else {
			// Error:
			// Probably couldn't create file...
			return ( $entryFile );
		}
	}
	
	function delete_entry ( $entry, $path ) {
		// Delete Entry File
		if ( file_exists( $path . $entry . '.txt' ) ) {
			$ok = sb_delete_file( $path . $entry . '.txt' );
		}
		if ( file_exists( $path . $entry . '.txt.gz' ) ) {
			$ok = sb_delete_file( $path . $entry . '.txt.gz' );
		}
		
		// Delete Comment Directory
		$dirpath = $path . $entry;
		
		// Get listing of files in folder.
		$file_array = sb_folder_listing( $dirpath . '/', array( '.txt', '.gz' ) );
		if ( count( $file_array ) == 0 ) {
			// Directory is empty, delete it...
			sb_delete_directory( $dirpath );
		} else {
			for ( $i = 0; $i < count( $file_array ); $i++ ) {
				sb_delete_file( $dirpath . '/' . $file_array[$i] );
				delete_most_recent( $file_array[$i] );
			}
			sb_delete_directory( $dirpath );
		}
		
		return ( $ok );
	}
	
	function write_static_entry ( $blog_subject, $blog_text, $updateFile ) {
		// Save new static entry or update old static entry
		//
		global $blog_config;
		
		clearstatcache();
			
		if ( $updateFile != NULL ) {
			// Updating a static entry
			//
			// We need to grab the date of the old entry
			if ( file_exists( 'content/static/' . $updateFile . '.txt' ) ) {
				$oldEntryFile = 'content/static/' . $updateFile . '.txt';
			} elseif ( file_exists( 'content/static/' . $updateFile . '.txt.gz' ) ) {
				$oldEntryFile = 'content/static/' . $updateFile . '.txt.gz';
			}
			$str = sb_read_file( $oldEntryFile );
			list( $temp_subject, $blog_date, $temp_text ) = explode( '|', $str );
			
			if ( $blog_config[ 'blog_enable_gzip_txt' ] ) {
				if ( file_exists( 'content/static/' . $updateFile . '.txt' ) ) {
					sb_delete_file( 'content/static/' . $updateFile . '.txt' );
				}
				$entryFile = $updateFile . '.txt.gz';
			} else {
				if ( file_exists( 'content/static/' . $updateFile . '.txt.gz' ) ) {
					sb_delete_file( 'content/static/' . $updateFile . '.txt.gz' );
				}
				$entryFile = $updateFile . '.txt';
			}
			
			// (We're just going to assume that the files and dirs exist...)
		} else {
			// Create a static entry
			//
			// This is going to be a new static entry.
			// We might need to create some directories.
			//
			// The directory and file structure is:
			// 'content/static/entryYYMMDD-HHMMSS.txt'
			$blog_date = time();
			
			if (!file_exists('content')) {
				$oldumask = umask(0);
				$ok = mkdir('content', 0777 );
				umask($oldumask);
			}
			
			if (!file_exists('content/static')) {
				$oldumask = umask(0);
				$ok = mkdir('content/static', 0777 );
				umask($oldumask);
				if (!$ok) {
					return ( 'Couldn\'t creating directory: '.'content/static' );
				}
			}
			
			$stamp = date('ymd-His');
			if ( $blog_config[ 'blog_enable_gzip_txt' ] ) {
				$entryFile = 'static'.$stamp.'.txt.gz';
			} else {
				$entryFile = 'static'.$stamp.'.txt';
			}
			
		}
		
		// Save the file
		$array = array( clean_post_text( $blog_subject ), $blog_date, clean_post_text( $blog_text ) );
		$str = implode('|', $array);
		
		$result = sb_write_file( 'content/static/'.$entryFile, $str );
		
		if ( $result ) {
			if ( $updateFile == NULL ) {
				// Add link to menu
				write_link( clean_post_text( $blog_subject ), 'static.php?page='.sb_strip_extension( $entryFile ), '' );
			} else {
				// Rename link
				$link_url = 'static.php?page='.sb_strip_extension( $entryFile );
				$result = sb_read_file( 'config/links.txt' );
				
				// Append new links.
				if ( $result ) {
					$array = explode('|', $result);
					for ( $i = 0; $i < count( $array ); $i++ ) {
						if ( $link_url == $array[$i] ) {
							write_link( clean_post_text( $blog_subject ), 'static.php?page='.sb_strip_extension( $entryFile ), $i-1 );
							break;
						}
					}
				}
			}
			return ( true );
		} else {
			// Error:
			// Probably couldn't create file...
			return ( 'content/static/'.$entryFile );
		}
	}
	
	// ----------------------
	// Blog Comment Functions
	// ----------------------

	function write_comment ( $y, $m, $entry, $comment_name, $comment_text) {
		// Save new entry or update old entry
		//
		global $blog_config;
		
		clearstatcache();
		
		// We're going to assume that the y and m directories exist...
		$basedir = 'content/';
		$dir = $basedir.$y.'/'.$m.'/'.$entry;
		
		if (!file_exists($dir)) {
			$oldumask = umask(0);
			$ok = mkdir($dir, 0777 );
			umask($oldumask);
			if (!$ok) {
				// There is a bug in some versions of PHP that will
				// cause mkdir to fail if there is a trailing '/'.
				//
				// Thanks to Matt - http://agent.chaosnet.org
				return ( $dir );
			}
		}
		
		$dir = $dir . '/';
		
		$comment_date = time();
		
		$stamp = date('ymd-His');
		if ( $blog_config[ 'blog_enable_gzip_txt' ] ) {
			$entryFile = $dir.'comment'.$stamp.'.txt.gz';
		} else {
			$entryFile = $dir.'comment'.$stamp.'.txt';
		}
		
		// Save the file
		$array = array( clean_post_text( $comment_name ), $comment_date, clean_post_text( $comment_text ) );
		$str = implode('|', $array);
		
		$result = sb_write_file( $entryFile, $str );
			
		if ( $result ) {
			// Update Most Recent List
	 		add_most_recent( 'comment'.$stamp, $y, $m, $entry );
	 		
	 		if ( $blog_config[ 'blog_email_notification' ] ) {
				// Send Email Notification:
				$subject="New comment posted at " . $blog_config[ 'blog_title' ];
				$body="Posted by: " . $comment_name . "<br />\n";
				
				if ( dirname($_SERVER['PHP_SELF']) == '\\' ) {
					// Hosted at root.
					$base_url = 'http://'.$_SERVER['HTTP_HOST'].'/';
				} else {
					// Hosted in sub-directory.
					$base_url = 'http://'.$_SERVER['HTTP_HOST'].dirname($_SERVER['PHP_SELF']).'/';
				}
				
				$body=$body . "<a href=\"" . $base_url . "comments.php?y=" . $y . "&m=" . $m . "&entry=" . $entry . "\">" . $base_url . "comments.php?y=" . $y . "&m=" . $m . "&entry=" . $entry . "</a><br /><br />\n\n";
				$body=$body . "<i>On " . format_date( $comment_date ) . ", " . $comment_name . " wrote:</i><br /><br />\n\n" . blog_to_html( $comment_text, false, false );
				sb_mail( $blog_config[ 'blog_email' ], $blog_config[ 'blog_email' ], $subject, $body, false );
	 		}
	 		
			return ( true );
		} else {
			// Error:
			// Probably couldn't create file...
			return ( $entryFile );
		}
	}
	
	function delete_comment ( $filepath ) {
		// Delete a comment. Also, delete the whole comment folder if it was the only comment.
		//
		
		// Delete comment file
		$ok = sb_delete_file( $filepath );
		
		// Trim off filename and leave path to last directory.
		$dirpath = $filepath;
		$pos = strrpos( $dirpath, '/' );
		if ($pos !== false) {
			$dirpath = substr( $dirpath, 0, $pos );
			
			// Get listing of files in folder.
			$file_array = sb_folder_listing( $dirpath . '/', array( '.txt', '.gz' ) );
			if ( count( $file_array ) == 0 ) {
				// Directory is empty, delete it...
				sb_delete_directory( $dirpath );
			} else {
				if ( $file_array[0] == 'view_counter.txt' ) {
					// There is one file and it's the 'view_counter.txt' file.
					//
					// Delete it and then delete the directory.
					sb_delete_file( $dirpath . '/view_counter.txt' );
					$result = sb_delete_directory( $dirpath );	
				}
			}
		}
		
		if ( $ok ) {
			delete_most_recent( $filepath );
		}
		
		return ( $ok );
	}
	
	// -------------------------------
	// Date Config / Display Functions
	// -------------------------------
	
	function write_dateFormat ( $array ) {
		// Save date/time format information to file.
		//
						
		$str = implode('|', $array);
		
		if (!file_exists('config')) {
			$oldumask = umask(0);
			$ok = mkdir('config', 0777 );
			umask($oldumask);
		}
		
		$filename = 'config/date_format.txt';
		$result = sb_write_file( $filename, $str );
		
		if ( $result ) {
			return ( true );
		} else {
			// Error:
			// Probably couldn't create file...
			return ( $filename );
		}
	}
	
	function read_dateFormat () {
		
		$dateArray = array();
		$dateArray['lDate_slotOne'] = 'weekday';
		$dateArray['lDate_slotOneSeparator'] = ', ';
		$dateArray['lDate_slotTwo'] = 'month';
		$dateArray['lDate_slotTwoSeparator'] = ' ';
		$dateArray['lDate_slotThree'] = 'day';
		$dateArray['lDate_slotThreeSeparator'] = ', ';
		$dateArray['lDate_slotFour'] = 'year';
		$dateArray['lDate_slotFourSeparator'] = '';
		$dateArray['lDate_leadZeroDay'] = '';
		$dateArray['sDate_order'] = 'Month/Day/Year';
		$dateArray['sDate_separator'] = '/';
		$dateArray['sDate_leadZeroDay'] = '';
		$dateArray['sDate_leadZeroMonth'] = '';
		$dateArray['sDate_fullYear'] = '';
		$dateArray['time_clockFormat'] = '12';
		$dateArray['time_leadZeroHour'] = 'on';
		$dateArray['time_AM'] = ' AM';
		$dateArray['time_PM'] = ' PM';
		$dateArray['time_separator'] = ':';
		$dateArray['eFormat_slotOne'] = 'long';
		$dateArray['eFormat_separator'] = ', ';
		$dateArray['eFormat_slotTwo'] = 'time';
		$dateArray['server_offset'] = '0';
		$dateArray['mFormat'] = 'short';
		
		$contents = sb_read_file( 'config/date_format.txt' );
			
		if ( $contents ) {
			$array = explode('|', $contents);
			
			$dateArray['lDate_slotOne'] = $array[ 0 ];
			$dateArray['lDate_slotOneSeparator'] = $array[ 1 ];
			$dateArray['lDate_slotTwo'] = $array[ 2 ];
			$dateArray['lDate_slotTwoSeparator'] = $array[ 3 ];
			$dateArray['lDate_slotThree'] = $array[ 4 ];
			$dateArray['lDate_slotThreeSeparator'] = $array[ 5 ];
			$dateArray['lDate_slotFour'] = $array[ 6 ];
			$dateArray['lDate_slotFourSeparator'] = $array[ 7 ];
			$dateArray['lDate_leadZeroDay'] = $array[ 8 ];
			$dateArray['sDate_order'] = $array[ 9 ];
			$dateArray['sDate_separator'] = $array[ 10 ];
			$dateArray['sDate_leadZeroDay'] = $array[ 11 ];
			$dateArray['sDate_leadZeroMonth'] = $array[ 12 ];
			$dateArray['sDate_fullYear'] = $array[ 13 ];
			$dateArray['time_clockFormat'] = $array[ 14 ];
			$dateArray['time_leadZeroHour'] = $array[ 15 ];
			$dateArray['time_AM'] = $array[ 16 ];
			$dateArray['time_PM'] = $array[ 17 ];
			$dateArray['time_separator'] = $array[ 18 ];
			$dateArray['eFormat_slotOne'] = $array[ 19 ];
			$dateArray['eFormat_separator'] = $array[ 20 ];
			$dateArray['eFormat_slotTwo'] = $array[ 21 ];
			$dateArray['server_offset'] = $array[ 22 ];
			$dateArray['mFormat'] = $array[ 23 ];
		}
		
		return ( $dateArray );	
	}
	
	function format_date ( $time_stamp ) {
		if ( strpos( $time_stamp, ',' ) !== false ) {
			// This is a hack for compatibility with the time
			// format from versions < 0.3.3. In 0.3.3 we switched
			// to the unix timestamp for storing times.
			//
			// Before that it was in this format:
			//   date( 'F j, Y, g:i a', $time_stamp );
			//   'May 10, 2004, 3:57 pm'
			$time_stamp = str_replace( ',', '', $time_stamp );
			$time_stamp = strtotime( $time_stamp );
		}
		
		// Read config information from file.
		
		$dateArray = read_dateFormat();
		
		$time_stamp = $time_stamp + ( intval( $dateArray['server_offset'] ) * 60 * 60);
		
		// Long Date
		$date_long = '';
		$date_long = $date_long . date_convert( $dateArray['lDate_slotOne'], $dateArray['lDate_leadZeroDay'], 'off', 'on', $time_stamp );
		$date_long = $date_long . $dateArray['lDate_slotOneSeparator'];
		$date_long = $date_long . date_convert( $dateArray['lDate_slotTwo'], $dateArray['lDate_leadZeroDay'], 'off', 'on', $time_stamp );
		$date_long = $date_long . $dateArray['lDate_slotTwoSeparator'];
		$date_long = $date_long . date_convert( $dateArray['lDate_slotThree'], $dateArray['lDate_leadZeroDay'], 'off', 'on', $time_stamp );
		$date_long = $date_long . $dateArray['lDate_slotThreeSeparator'];
		$date_long = $date_long . date_convert( $dateArray['lDate_slotFour'], $dateArray['lDate_leadZeroDay'], 'off', 'on', $time_stamp );
		$date_long = $date_long . $dateArray['lDate_slotFourSeparator'];
		
		// Short Date
		$date_short = '';
		$separator = $dateArray['sDate_separator'];
		$leading_zero_day = $dateArray['sDate_leadZeroDay'];
		$leading_zero_month = $dateArray['sDate_leadZeroMonth'];
		$full_century = $dateArray['sDate_fullYear'];
		switch ( $dateArray['sDate_order'] ) {
			case 'Month/Day/Year':
				$date_short = $date_short . date_convert( 'month_decimal', $leading_zero_day, $leading_zero_month, $full_century, $time_stamp );
				$date_short = $date_short . $separator;
				$date_short = $date_short . date_convert( 'day', $leading_zero_day, $leading_zero_month, $full_century, $time_stamp );
				$date_short = $date_short . $separator;
				$date_short = $date_short . date_convert( 'year', $leading_zero_day, $leading_zero_month, $full_century, $time_stamp );
				break;
			case 'Day/Month/Year':
				$date_short = $date_short . date_convert( 'day', $leading_zero_day, $leading_zero_month, $full_century, $time_stamp );
				$date_short = $date_short . $separator;
				$date_short = $date_short . date_convert( 'month_decimal', $leading_zero_day, $leading_zero_month, $full_century, $time_stamp );
				$date_short = $date_short . $separator;
				$date_short = $date_short . date_convert( 'year', $leading_zero_day, $leading_zero_month, $full_century, $time_stamp );
				break;
			case 'Year/Month/Day':
				$date_short = $date_short . date_convert( 'year', $leading_zero_day, $leading_zero_month, $full_century, $time_stamp );
				$date_short = $date_short . $separator;
				$date_short = $date_short . date_convert( 'month_decimal', $leading_zero_day, $leading_zero_month, $full_century, $time_stamp );
				$date_short = $date_short . $separator;
				$date_short = $date_short . date_convert( 'day', $leading_zero_day, $leading_zero_month, $full_century, $time_stamp );
				break;
			case 'Month/Year/Day':
				$date_short = $date_short . date_convert( 'month_decimal', $leading_zero_day, $leading_zero_month, $full_century, $time_stamp );
				$date_short = $date_short . $separator;
				$date_short = $date_short . date_convert( 'year', $leading_zero_day, $leading_zero_month, $full_century, $time_stamp );
				$date_short = $date_short . $separator;
				$date_short = $date_short . date_convert( 'day', $leading_zero_day, $leading_zero_month, $full_century, $time_stamp );
				break;
			case 'Day/Year/Month':
				$date_short = $date_short . date_convert( 'day', $leading_zero_day, $leading_zero_month, $full_century, $time_stamp );
				$date_short = $date_short . $separator;
				$date_short = $date_short . date_convert( 'year', $leading_zero_day, $leading_zero_month, $full_century, $time_stamp );
				$date_short = $date_short . $separator;
				$date_short = $date_short . date_convert( 'month_decimal', $leading_zero_day, $leading_zero_month, $full_century, $time_stamp );
				break;
			case 'Year/Day/Month':
				$date_short = $date_short . date_convert( 'year', $leading_zero_day, $leading_zero_month, $full_century, $time_stamp );
				$date_short = $date_short . $separator;
				$date_short = $date_short . date_convert( 'day', $leading_zero_day, $leading_zero_month, $full_century, $time_stamp );
				$date_short = $date_short . $separator;
				$date_short = $date_short . date_convert( 'month_decimal', $leading_zero_day, $leading_zero_month, $full_century, $time_stamp );
				break;
		}
		
		// Time View
		$time_str = '';
		$time_clockFormat = $dateArray['time_clockFormat'];
		$leading_zero_hour = $dateArray['time_leadZeroHour'];
		$before_noon = $dateArray['time_AM'];
		$after_noon = $dateArray['time_PM'];
		$separator = $dateArray['time_separator'];
		
		if ( $time_clockFormat == "24" ) {
			if ( $leading_zero_hour == 'on' ) {
				$time_str = $time_str . date( 'H', $time_stamp ) . $separator . date( 'i', $time_stamp );
			} else {
				$time_str = $time_str . date( 'G', $time_stamp ) . $separator . date( 'i', $time_stamp );
			}
		} else {
			if ( $leading_zero_hour == 'on' ) {
				$time_str = $time_str . date( 'h', $time_stamp ) . $separator . date( 'i', $time_stamp );
				if ( date( 'a', $time_stamp ) == 'am' ) {
					$time_str = $time_str . $before_noon;
				} else {
					$time_str = $time_str . $after_noon;
				}
			} else {
				$time_str = $time_str . date( 'g', $time_stamp ) . $separator . date( 'i', $time_stamp );
				if ( date( 'a', $time_stamp ) == 'am' ) {
					$time_str = $time_str . $before_noon;
				} else {
					$time_str = $time_str . $after_noon;
				}
			}
		}
		
		// Put it all together...
		$str = '';
		switch ( $dateArray['eFormat_slotOne'] ) {
			case 'long':
				$str = $str . $date_long;
				break;
			case 'short':
				$str = $str . $date_short;
				break;
			case 'time':
				$str = $str . $time_str;
				break;
			case 'none':
				break;
		}
					
		$str = $str . $dateArray['eFormat_separator'];
		
		switch ( $dateArray['eFormat_slotTwo'] ) {
			case 'long':
				$str = $str . $date_long;
				break;
			case 'short':
				$str = $str . $date_short;
				break;
			case 'time':
				$str = $str . $time_str;
				break;
			case 'none':
				break;
		}
		
		return ( clean_post_text( $str ) );
	}
	
	function format_date_menu ( $time_stamp ) {
		if ( strpos( $time_stamp, ',' ) !== false ) {
			// This is a hack for compatibility with the time
			// format from versions < 0.3.3. In 0.3.3 we switched
			// to the unix timestamp for storing times.
			//
			// Before that it was in this format:
			//   date( 'F j, Y, g:i a', $time_stamp );
			//   'May 10, 2004, 3:57 pm'
			$time_stamp = str_replace( ',', '', $time_stamp );
			$time_stamp = strtotime( $time_stamp );
		}
		
		// Read config information from file.
		
		$dateArray = read_dateFormat();
		
		$time_stamp = $time_stamp + ( intval( $dateArray['server_offset'] ) * 60 * 60);
		
		// Long Date
		$date_long = '';
		$date_long = $date_long . date_convert( $dateArray['lDate_slotOne'], $dateArray['lDate_leadZeroDay'], 'off', 'on', $time_stamp );
		$date_long = $date_long . $dateArray['lDate_slotOneSeparator'];
		$date_long = $date_long . date_convert( $dateArray['lDate_slotTwo'], $dateArray['lDate_leadZeroDay'], 'off', 'on', $time_stamp );
		$date_long = $date_long . $dateArray['lDate_slotTwoSeparator'];
		$date_long = $date_long . date_convert( $dateArray['lDate_slotThree'], $dateArray['lDate_leadZeroDay'], 'off', 'on', $time_stamp );
		$date_long = $date_long . $dateArray['lDate_slotThreeSeparator'];
		$date_long = $date_long . date_convert( $dateArray['lDate_slotFour'], $dateArray['lDate_leadZeroDay'], 'off', 'on', $time_stamp );
		$date_long = $date_long . $dateArray['lDate_slotFourSeparator'];
		
		// Short Date
		$date_short = '';
		$separator = $dateArray['sDate_separator'];
		$leading_zero_day = $dateArray['sDate_leadZeroDay'];
		$leading_zero_month = $dateArray['sDate_leadZeroMonth'];
		$full_century = $dateArray['sDate_fullYear'];
		switch ( $dateArray['sDate_order'] ) {
			case 'Month/Day/Year':
				$date_short = $date_short . date_convert( 'month_decimal', $leading_zero_day, $leading_zero_month, $full_century, $time_stamp );
				$date_short = $date_short . $separator;
				$date_short = $date_short . date_convert( 'day', $leading_zero_day, $leading_zero_month, $full_century, $time_stamp );
				$date_short = $date_short . $separator;
				$date_short = $date_short . date_convert( 'year', $leading_zero_day, $leading_zero_month, $full_century, $time_stamp );
				break;
			case 'Day/Month/Year':
				$date_short = $date_short . date_convert( 'day', $leading_zero_day, $leading_zero_month, $full_century, $time_stamp );
				$date_short = $date_short . $separator;
				$date_short = $date_short . date_convert( 'month_decimal', $leading_zero_day, $leading_zero_month, $full_century, $time_stamp );
				$date_short = $date_short . $separator;
				$date_short = $date_short . date_convert( 'year', $leading_zero_day, $leading_zero_month, $full_century, $time_stamp );
				break;
			case 'Year/Month/Day':
				$date_short = $date_short . date_convert( 'year', $leading_zero_day, $leading_zero_month, $full_century, $time_stamp );
				$date_short = $date_short . $separator;
				$date_short = $date_short . date_convert( 'month_decimal', $leading_zero_day, $leading_zero_month, $full_century, $time_stamp );
				$date_short = $date_short . $separator;
				$date_short = $date_short . date_convert( 'day', $leading_zero_day, $leading_zero_month, $full_century, $time_stamp );
				break;
			case 'Month/Year/Day':
				$date_short = $date_short . date_convert( 'month_decimal', $leading_zero_day, $leading_zero_month, $full_century, $time_stamp );
				$date_short = $date_short . $separator;
				$date_short = $date_short . date_convert( 'year', $leading_zero_day, $leading_zero_month, $full_century, $time_stamp );
				$date_short = $date_short . $separator;
				$date_short = $date_short . date_convert( 'day', $leading_zero_day, $leading_zero_month, $full_century, $time_stamp );
				break;
			case 'Day/Year/Month':
				$date_short = $date_short . date_convert( 'day', $leading_zero_day, $leading_zero_month, $full_century, $time_stamp );
				$date_short = $date_short . $separator;
				$date_short = $date_short . date_convert( 'year', $leading_zero_day, $leading_zero_month, $full_century, $time_stamp );
				$date_short = $date_short . $separator;
				$date_short = $date_short . date_convert( 'month_decimal', $leading_zero_day, $leading_zero_month, $full_century, $time_stamp );
				break;
			case 'Year/Day/Month':
				$date_short = $date_short . date_convert( 'year', $leading_zero_day, $leading_zero_month, $full_century, $time_stamp );
				$date_short = $date_short . $separator;
				$date_short = $date_short . date_convert( 'day', $leading_zero_day, $leading_zero_month, $full_century, $time_stamp );
				$date_short = $date_short . $separator;
				$date_short = $date_short . date_convert( 'month_decimal', $leading_zero_day, $leading_zero_month, $full_century, $time_stamp );
				break;
		}
		
		// Time View
		$time_str = '';
		$time_clockFormat = $dateArray['time_clockFormat'];
		$leading_zero_hour = $dateArray['time_leadZeroHour'];
		$before_noon = $dateArray['time_AM'];
		$after_noon = $dateArray['time_PM'];
		$separator = $dateArray['time_separator'];
		
		if ( $time_clockFormat == "24" ) {
			if ( $leading_zero_hour == 'on' ) {
				$time_str = $time_str . date( 'H', $time_stamp ) . $separator . date( 'i', $time_stamp );
			} else {
				$time_str = $time_str . date( 'G', $time_stamp ) . $separator . date( 'i', $time_stamp );
			}
		} else {
			if ( $leading_zero_hour == 'on' ) {
				$time_str = $time_str . date( 'h', $time_stamp ) . $separator . date( 'i', $time_stamp );
				if ( date( 'a', $time_stamp ) == 'am' ) {
					$time_str = $time_str . $before_noon;
				} else {
					$time_str = $time_str . $after_noon;
				}
			} else {
				$time_str = $time_str . date( 'g', $time_stamp ) . $separator . date( 'i', $time_stamp );
				if ( date( 'a', $time_stamp ) == 'am' ) {
					$time_str = $time_str . $before_noon;
				} else {
					$time_str = $time_str . $after_noon;
				}
			}
		}
		
		// Put it all together...
		$str = '';
		switch ( $dateArray['mFormat'] ) {
			case 'long':
				$str = $str . $date_long;
				break;
			case 'short':
				$str = $str . $date_short;
				break;
		}
		
		return ( clean_post_text( $str ) );
	}
		
	function date_convert( $val, $leading_zero_day, $leading_zero_month, $full_century, $time_stamp ) {
		// Return string dates in the correct format.
		//
		
		$str = '';
		if ( $val == 'weekday' ) {
			// Monday
			$str = strftime( '%A', $time_stamp );
		} else if ( $val == 'month' ) {
			// January
			$str = strftime( '%B', $time_stamp );
		} else if ( $val == 'month_decimal' ) {
			if ( $leading_zero_month == 'on' ) {
				$str = date( 'm', $time_stamp );
			} else {
				$str = date( 'n', $time_stamp );
			}
		} else if ( $val == 'day' ) {
			if ( $leading_zero_day == 'on' ) {
				$str = date( 'd', $time_stamp );
			} else {
				$str = date( 'j', $time_stamp );
			}
		} else if ( $val == 'year' ) {
			if ( $full_century == 'on' ) {
				$str = date( 'Y', $time_stamp );
			} else {
				$str = date( 'y', $time_stamp );
			}
		} else if ( $val == 'none' ) {
			$str = '';
		}
		
		return $str;
	}
	
	// ----------------------
	// Blog Display Functions
	// ----------------------
	
	function search ( $search_str ) {
		// Search in contents feature a list of keywords separated by spaces
		// (c) 2004 Javier Gutirrez Chamorro (Guti), guti <at> ya <dot> com
		//
		// Simple PHP Version: 0.3.7
		// Search Version:   0.3.7
		//
	
		// Read entries by month, year and/or day. Generate HTML output.
		//
		// Used for the main Index page.
		global $lang_string, $blog_config, $user_colors;
		
		$output_str = '';
		
		$entry_file_array = blog_entry_listing();
		
		// Flip entry order
		if ( $blog_config[ 'blog_entry_order' ] == 'old_to_new' ) {
			$entry_file_array = array_reverse( $entry_file_array );
		}
		
		$words=@split( ' ', $search_str );
		
		if ( strlen( $search_str ) > 1) {
			for ( $i = 0; $i < count( $words ); $i++) {
				$words[$i] = strtoupper( trim($words[$i] ) );
			}
			
			// Loop through entry files
			for ( $i = 0; $i < count( $entry_file_array ); $i++ ) {
				list( $entry_filename, $year_dir, $month_dir ) = explode( '|', $entry_file_array[ $i ] );
				$contents=sb_read_file( 'content/' . $year_dir . '/' . $month_dir . '/' . $entry_filename );
				$found=true;
				$j=0;
				while ( ( $j<count( $words ) ) && ( $found ) ) {
					$text=strtoupper( $contents );
					if ( $words[$j] !== '' ) {
						$found = $found && ( strpos( $text, $words[$j] ) !== false );
					}
					$j++;
				}
				if ( $found ) {
					list( $blog_subject, $blog_date, $blog_text ) = explode('|', ( $contents ) );
					$output_str = $output_str . '<a href="index.php?entry=' . sb_strip_extension( $entry_filename ) . '" title="' . format_date( $blog_date ) . '">' . $blog_subject . "</a><br />";
				}
				
				// Search Comments
				if ( $blog_config[ 'blog_enable_comments' ] == true ) {
					$comment_file_array = sb_folder_listing( 'content/' . $year_dir . '/' . $month_dir . '/' . sb_strip_extension( $entry_filename ) . '/', array( '.txt', '.gz' ) );
					if ( $comment_file_array > 0 ) {
						for ( $k = 0; $k < count( $comment_file_array ); $k++ ) {
							$comment_filename =  $comment_file_array[ $k ];
							$contents_comment = sb_read_file( 'content/' . $year_dir . '/' . $month_dir . '/' . sb_strip_extension( $entry_filename ) . '/' . $comment_filename );
							$found_in_comment = true;
							$l = 0;
							while ( ( $l< count( $words ) ) && ( $found_in_comment ) ) {
								$text = strtoupper( $contents_comment );
								if ( $words[$l] !== '' ) {
									$found_in_comment = $found_in_comment && ( strpos( $text, $words[$l] ) !== false );
								}
								$l++;
							}
							if ( $found_in_comment ) {
								if ( $found == false ) {
									list( $blog_subject, $blog_date, $blog_text ) = explode('|', ( $contents ) );
									$output_str = $output_str . $blog_subject . "<br />";
								}
							
								list( $comment_author, $comment_date, $comment_text ) = explode('|', ( $contents_comment ) );
								
								if ( $blog_config[ 'blog_comments_popup' ] == 1 ) {
									$output_str = $output_str . '&nbsp;&nbsp;&nbsp;<a href="javascript:openpopup(\'comments.php?y='.$year_dir.'&m='.$month_dir.'&entry='. sb_strip_extension($entry_filename).'\',500,500,true)">' . $comment_author . "</a><br />";
								} else {
									$output_str = $output_str . '&nbsp;&nbsp;&nbsp;<a href="comments.php?y=' . $year_dir . '&m=' . $month_dir . '&entry=' . sb_strip_extension( $entry_filename ) . '" title="' . format_date( $comment_date ) . '">' . $comment_author . "</a><br />";
								}
								
							}
						}
					}
				}
					
					
			}
		}
		
		return ( $output_str );
	}
	
	function get_entry_by_file ( $entry_id ) {
		// Read an entry from disk and create the HTML
		//
		// Used by 'get_comments' function.
		//
		global $blog_content, $blog_subject, $blog_text, $blog_date, $user_colors, $logged_in;
		
		if ( file_exists( $entry_id . '.txt' ) ) {
			$filename = $entry_id . '.txt';
		} elseif ( file_exists( $entry_id . '.txt.gz' ) ) {
			$filename = $entry_id . '.txt.gz';
		}
		$str = sb_read_file( $filename );
		
		list( $blog_subject, $blog_date, $blog_text ) = explode('|', $str );
		
		$entry_array = array();
		$entry_array['subject'] = blog_to_html( $blog_subject, false, false );
		$entry_array['date'] = blog_to_html( format_date( $blog_date ), false, false );
		$entry_array['entry'] = blog_to_html( $blog_text, false, false ) . '<br clear="all">';
		
		$blog_content = theme_blogentry( $entry_array, $logged_in );
		
		return ( $blog_content );
	}
	
	function preview_entry ( $blog_subject, $blog_text ) {
		// Function to preview an entry before saving it to disk
		//
		// Just going through the motions...
		global $blog_content, $user_colors, $logged_in;
		
		$array = array( clean_post_text( $blog_subject ), time(), clean_post_text( $blog_text ) );
		$str = implode('|', $array);
		
		list( $blog_subject, $blog_date, $blog_text ) = explode( '|', $str );
		
		$entry_array = array();
		$entry_array['subject'] = blog_to_html( $blog_subject, false, false );
		$entry_array['date'] = blog_to_html( format_date( $blog_date ), false, false );
		$entry_array['entry'] = blog_to_html( $blog_text, false, false ) . '<br clear="all">';
		
		$blog_content = theme_blogentry( $entry_array, $logged_in );
		
		return ( $blog_content );
	}
	
	function get_static_entry_by_file ( $entry_id ) {
		// Read an entry from disk and create the HTML
		//
		global $blog_content, $blog_subject, $blog_text, $blog_date, $user_colors, $logged_in;
		
		if ( file_exists( 'content/static/' . $entry_id . '.txt' ) ) {
			$filename = $entry_id . '.txt';
		} elseif ( file_exists( 'content/static/' . $entry_id . '.txt.gz' ) ) {
			$filename = $entry_id . '.txt.gz';
		}
		$str = sb_read_file( 'content/static/'.$filename );
		
		list( $blog_subject, $blog_date, $blog_text ) = explode('|', $str );
		
		$entry_array = array();
		$entry_array['subject'] = blog_to_html( $blog_subject, false, false );
		$entry_array['date'] = blog_to_html( format_date( $blog_date ), false, false );
		$entry_array['entry'] = blog_to_html( $blog_text, false, false ) . '<br clear="all">';
		
		$blog_content = theme_staticentry( $entry_array, $logged_in );
		
		return ( $blog_content );
	}
	
	function get_comments ( $y, $m, $entry, $logged_in ) {
		global $blog_content, $blog_config, $lang_string, $user_colors;
		
		$filename = 'content/'.$y.'/'.$m.'/'.$entry;
		
		$blog_content = get_entry_by_file( $filename ) . '<br />';
		
		// Comments
		$basedir = 'content/';
		$dir = $basedir.$y.'/'.$m.'/'.$entry.'/';
		$file_array = sb_folder_listing( $dir, array( '.txt', '.gz' ) );
		if ( $blog_config[ 'blog_comment_order' ] == 'new_to_old' ) {
			$file_array = array_reverse( $file_array );
		}
		
		$view_counter = 1;
		$contents = array();
		for ( $i = 0; $i < count( $file_array ); $i++ ) {
			if ( $file_array[$i] == 'view_counter.txt' ) {
				$view_counter = intval( sb_read_file( $dir . $file_array[$i] ) ) + 1;
			} else {
				array_push( $contents, array( 'text' => sb_read_file( $dir . $file_array[$i] ), 'entry' => $file_array[$i] ) );
			}
		}
		
		// Store Counter
		sb_write_file( $dir . 'view_counter.txt' , $view_counter );
		
		if ( $contents ) {
				// Display comments Oldest to Newest to. Oldest Comments will be at the top of the page.
				for ( $i = 0; $i <= count( $contents ) - 1; $i++ ) {
				
					list( $comment_name, $comment_date, $comment_text ) = explode('|', ( $contents[$i]['text'] ) );
					
					$entry_array = array();
					$entry_array['subject'] = blog_to_html( $comment_name, true, false );
					$entry_array['date'] = blog_to_html( format_date( $comment_date ), true, false );
					$entry_array['entry'] = blog_to_html( $comment_text, true, false ) . '<br clear="all">';
					
					$entry_array['logged_in'] = $logged_in;
					
					// Author
					$comment = $contents[$i]['entry'];
					if ( $logged_in ) {
						$entry_array['delete']['name'] = $lang_string['delete_btn'];
						$entry_array['delete']['url'] = 'comment_delete_cgi.php?y='.$y.'&m='.$m.'&entry='.$entry.'&comment='.$dir.$comment;
					}
					
					$entry_array['count'] = $i;
					$entry_array['maxcount'] = count( $contents ) - 1;
					
					$blog_content = $blog_content . theme_commententry( $entry_array );
				}
		}
		
		return $blog_content;
	}
	
	function blog_entry_listing () {
		// Return listing of all the blog entries in order
		// of newest to oldest.
		//
		// An improvement would be to Cache this list in a
		// file and only recalculate when you add or delete
		// an entry.
		clearstatcache();
		
		$basedir = 'content/';
		
		// YEAR directories
		$entry_array = array();
		$dir = $basedir;
		if ( is_dir( $dir ) ) {
			if ( $year_dir_handle = @opendir( $dir ) ) {
				while ( ( $year_dir = readdir( $year_dir_handle ) ) !== false ) {
					if ( is_dir( $dir . $year_dir . '/' ) ) {
						if ( $year_dir != '.' && $year_dir != '..' && $year_dir != 'static' ) {
						
							// MONTH directories
							
							if ( $month_dir_handle = @opendir( $dir . $year_dir . '/' ) ) {
								while ( ( $month_dir = readdir( $month_dir_handle ) ) !== false ) {
									if ( is_dir( $dir . $year_dir . '/' . $month_dir . '/' ) ) {
										if ( $month_dir != '.' && $month_dir != '..' ) {
										
											// ENTRIES
							
											if ( $entry_dir_handle = @opendir( $dir . $year_dir . '/' . $month_dir . '/' ) ) {
												while ( ( $entry_filename = readdir( $entry_dir_handle ) ) !== false ) {
													if ( is_file( $dir . $year_dir . '/' . $month_dir . '/' . $entry_filename ) ) {
														
														$ext = strtolower( strrchr( $entry_filename, '.' ) );
														if ( $ext == '.txt' || $ext == '.gz' ) {
															
															// Store Blog Entry Information
															array_push( $entry_array, implode( '|', array( $entry_filename, $year_dir, $month_dir ) ) );
														}
														
													}
												}
											}
											
											// END of ENTRIES
											
										}
									}
								}
							}
							
							// END of MONTH directories
							
						}
					}
				}
			}
		}
		
		sort( $entry_array );
		$entry_array = array_reverse( $entry_array );
		
		return( $entry_array );
	}
	
	function read_entries ( $m, $y, $d, $logged_in, $start_entry ) {		
		// Read entries by month, year and/or day. Generate HTML output.
		//
		// Used for the main Index page.
		global $lang_string, $blog_config, $user_colors;
		
		$entry_file_array = blog_entry_listing();
		
		// Loop through the $entry_file_array looking for the
		// first match. Note that $d could be NULL in which
		// case we get the first entry for the month.
		//
		// This is fine because this is actually what we
		// are looking for anyway.
		//
		// I'm just using a brute force method, I'm sure there
		// are better ways to do this... :)
		if ( $start_entry != NULL ) {
			$look_for = $start_entry;	
		} else {
			$look_for = 'entry' . $y . $m . $d;
		}
		
		$entry_index = 0;
		for ( $i = 0; $i < count( $entry_file_array ); $i++ ) {
			if ( $look_for == substr( $entry_file_array[ $i ], 0, strlen( $look_for ) ) ) {
				// MATCH!
				$entry_index = $i;
				break;
			}
		}
			
		// Grab the next X number of entries
		$file_array = array();
		for ( $i = $entry_index; $i < min( ( $blog_config[ 'blog_max_entries' ] + $entry_index ), count( $entry_file_array ) ); $i++ ) {
			array_push( $file_array, $entry_file_array[ $i ] );
		}
		
		// Store info for next and previous links...
		if ( $entry_index + $blog_config[ 'blog_max_entries' ] < count( $entry_file_array ) ) {
			$next_entry = $entry_file_array[ $entry_index + $blog_config[ 'blog_max_entries' ] ];
		} else {
			$next_entry = NULL;
		}
		if ( $entry_index - $blog_config[ 'blog_max_entries' ] >= 0 ) {
			$previous_entry = $entry_file_array[ $entry_index - $blog_config[ 'blog_max_entries' ] ];
		} else {
			$previous_entry = NULL;
		}
		
		// Flip entry order
		if ( $blog_config[ 'blog_entry_order' ] == 'old_to_new' ) {
			$file_array = array_reverse( $file_array );
		}
		
		// Read entry files
		$contents = array();
		for ( $i = 0; $i < count( $file_array ); $i++ ) {
			list( $entry_filename, $year_dir, $month_dir ) = explode( '|', $file_array[ $i ] );
			array_push( $contents, array( 'text' => sb_read_file( 'content/' . $year_dir . '/' . $month_dir . '/' . $entry_filename ), 'entry' => $entry_filename, 'year' => $year_dir, 'month' => $month_dir ) );
		}
		
		$blog_content = '';
		if ( $contents ) {
			for ( $i = 0; $i <= count( $contents ) - 1; $i++ ) {
				list( $blog_subject, $blog_date, $blog_text ) = explode('|', ( $contents[$i]['text'] ) );
				
				$entry_array = array();
				$entry_array['subject'] = blog_to_html( $blog_subject, false, false );
				$entry_array['date'] = blog_to_html( format_date( $blog_date ), false, false );
				
				// Author edit and delete
				$entry = sb_strip_extension( $contents[$i]['entry'] );
				$y = sb_strip_extension( $contents[$i]['year'] );
				$m = sb_strip_extension( $contents[$i]['month'] );
				
				if ( $logged_in ) {
					$entry_array['edit']['name'] = $lang_string['sb_edit'];
					$entry_array['edit']['url'] = 'preview_cgi.php?y='.$y.'&m='.$m.'&entry='.$entry;
					$entry_array['delete']['name'] = $lang_string['sb_delete'];
					$entry_array['delete']['url'] = 'delete.php?y='.$y.'&m='.$m.'&entry='.$entry;
				}
				
				// Comments link and count
				$entry_array['entry'] = blog_to_html( $blog_text, false, false ) . '<br clear="all">';
				
				$comment_path = 'content/'.$y.'/'.$m.'/'.$entry.'/';
				$comment_array = sb_folder_listing( $comment_path, array( '.txt', '.gz' ) );
				
				// Read view counter file
				$view_counter = 0;
				$comment_count = count( $comment_array );
				if ( in_array( 'view_counter.txt', $comment_array ) ) {
					$view_counter = intval( sb_read_file( $comment_path . 'view_counter.txt' ) );
					$comment_count = $comment_count - 1;
				}
				
				if ( $blog_config[ 'blog_enable_comments' ] == true ) {
					// Add comment buttons
					if ( $blog_config[ 'blog_comments_popup' ] == 1 ) {
						$entry_array['comment']['url'] = 'javascript:openpopup(\'comments.php?y='.$y.'&m='.$m.'&entry='.$entry.'\',500,500,true)';
					} else {
						$entry_array['comment']['url'] = 'comments.php?y='.$y.'&m='.$m.'&entry='.$entry;
					}
					
					if ( $comment_count == 0) {
						// [ add comment ]
						$entry_array['comment']['name'] = $lang_string['sb_add_comment_btn'];
					} else if ( $comment_count == 1) {
						// [ 1 comment ] (In Russian the number should come last.)
						if ( $lang_string['sb_comment_btn_number_first'] == true ) {
							$entry_array['comment']['name'] = $comment_count . ' ' . $lang_string['sb_comment_btn'];
						} else {
							$entry_array['comment']['name'] = $lang_string['sb_comment_btn'] . ' ' . $comment_count;
						}
					} else {
						// [ n comments ] (In Russian the number should come last.)
						if ( $lang_string['sb_comments_plural_btn_number_first'] == true ) {
							$entry_array['comment']['name'] = $comment_count . ' ' . $lang_string['sb_comments_plural_btn'];
						} else {
							$entry_array['comment']['name'] = $lang_string['sb_comments_plural_btn'] . ' ' . $comment_count;
						}
					}
					
					// Add view counter
					if ( $view_counter > 0 ) {
						if ( $view_counter == 1) {
							$entry_array['comment']['count'] = $lang_string['sb_view_counter_pre'] . $view_counter . $lang_string['sb_view_counter_post'];
						} else {
							$entry_array['comment']['count'] = $lang_string['sb_view_counter_plural_pre'] . $view_counter . $lang_string['sb_view_counter_plural_post'];
						}
					}
				}
				
				$entry_array['count'] = $i;
				$entry_array['maxcount'] = count( $contents ) - 1;
				$entry_array['logged_in'] = $logged_in;
				
				$blog_content = $blog_content . theme_blogentry( $entry_array );
			}
		}
		
		$blog_content = $blog_content . '<br>';
		
		if ( $previous_entry != NULL ) {
			list( $entry_filename, $year_dir, $month_dir ) = explode( '|', $previous_entry );
			$d = substr( $entry_filename, 9, 2 );
			$blog_content = $blog_content . ' <span style="float: left;"><a href="index.php?m=' . $month_dir . '&y=' . $year_dir . '&d=' . $d . '&entry=' . $entry_filename . '">' . $lang_string['nav_back'] . '</a></span> ';
		}
		
		if ( $next_entry != NULL ) {
			list( $entry_filename, $year_dir, $month_dir ) = explode( '|', $next_entry );
			$d = substr( $entry_filename, 9, 2 );
			$blog_content = $blog_content . ' <span style="float: right;"><a href="index.php?m=' . $month_dir . '&y=' . $year_dir . '&d=' . $d . '&entry=' . $entry_filename . '">' . $lang_string['nav_next'] . '</a></span><br> ';
		}
		
		return $blog_content;
	}
	
	function read_static_entry ( $entry_id, $logged_in ) {
		// Read entries by month and year. Generate HTML output.
		//
		// Used for the main Index page.
		global $lang_string, $blog_config, $user_colors;
		
		$dir = 'content/static/';
		
		
		if ( file_exists( 'content/static/' . $entry_id . '.txt' ) ) {
			$filename = $entry_id . '.txt';
		} elseif ( file_exists( 'content/static/' . $entry_id . '.txt.gz' ) ) {
			$filename = $entry_id . '.txt.gz';
		}
		
		$contents = array( 'text' => sb_read_file( 'content/static/' . $filename ), 'entry' => $entry_id );
		
		list( $blog_subject, $blog_date, $blog_text ) = explode('|', ( $contents['text'] ) );
		
		$entry_array = array();
		$entry_array['subject'] = blog_to_html( $blog_subject, false, false );
		$entry_array['date'] = blog_to_html( format_date( $blog_date ), false, false );
		$entry_array['entry'] = blog_to_html( $blog_text, false, false ) . '<br clear="all">';
		$entry_array['logged_in'] = $logged_in;
		
		// Author edit and delete
		$entry = sb_strip_extension( $contents['entry'] );
		if ( $logged_in ) {
			$entry_array['edit']['name'] = $lang_string['sb_edit'];
			$entry_array['edit']['url'] = 'preview_static_cgi.php?entry='.$entry;
			$entry_array['delete']['name'] = $lang_string['sb_delete'];
			$entry_array['delete']['url'] = 'delete_static.php?entry='.$entry;
		}
		
		$blog_content = theme_staticentry( $entry_array, $logged_in );
		
		return ( $blog_content );
	}
	
	// ----------------------
	// Image Upload Functions
	// ----------------------
	
	function image_list () {
		// Get a list of images in the image folder. Return HTML.
		//
		
		if (!file_exists('images')) {
			$oldumask = umask(0);
			$ok = mkdir('images', 0777 );
			umask($oldumask);
		}
			
		$dir = 'images/';
		// $contents = sb_folder_listing( $dir, array( '.jpg', '.gif', '.png' ) );
		$contents = sb_folder_listing( $dir, array() );
		
		$str = NULL;
		if ($contents) {
			for ( $i = 0; $i < count( $contents ); $i++ ) {
				$str = $str . '<a href='.$dir.$contents[$i].' target=_blank>'.$contents[$i].'</a><br />';
			}
		}
		
		return ( $str );
	}
	
	function image_dropdown () {
		// Get a list of images in the image folder. Return HTML.
		//
		global $theme_vars;
		
		$dir = 'images/';
		// $contents = sb_folder_listing( $dir, array( '.jpg', '.gif', '.png' ) );
		$contents = sb_folder_listing( $dir, array() );
		
		$str = NULL;
		if ($contents) {
			if ( count( $contents ) > 0 ) {
				$str = '<select name="image_list" onchange="ins_image_dropdown(this.form.blog_text,this.form.image_list);">';
				$str = $str . '<option label="--" value="--">--</option>';
				for ( $i = 0; $i < count( $contents ); $i++ ) {
					$str_url = $dir.$contents[$i];
					$img_size = @getimagesize( $str_url );
					if ( $img_size !== false ) {
						$width = $img_size[0];
						$height = $img_size[1];
						$max_image_width = $theme_vars['max_image_width'];
						if ( $width > $max_image_width ) {
							$height_resized = round( $height * ( $max_image_width / $width ) );
							$width_resized = $max_image_width;
							$str = $str . '<option label="'.$contents[$i].' - '.$width.' x '.$height.'" value="[img='.$dir.$contents[$i].' popup=true]">'.$contents[$i].'</option>';
						} else {
							$str = $str . '<option label="'.$contents[$i].' - '.$width.' x '.$height.'" value="[img='.$dir.$contents[$i].' popup=false]">'.$contents[$i].'</option>';
						}
					} else {
						$str = $str . '<option label="'.$contents[$i].'" value="[img='.$dir.$contents[$i].']">'.$contents[$i].'</option>';
					}
				}
				$str = $str . '</select>';
			}
		}
		
		return ( $str );
	}
	
	// ----------------------
	// Menu & Links Functions
	// ----------------------
	
	function get_latest_entry () {
		// Figure out the date of the last entry. Set default year and month global values.
		// This is done so we're not displaying an empty page. If it's April and the last
		// entry was March, then we want to show March's entries (not April...)
		//
		// Returns nothing but sets $GLOBALS['month'] and $GLOBALS['year']
		//
		clearstatcache();
		
		$basedir = 'content/';
		
		$latest_y = NULL;
		$latest_m = NULL;
		$latest_d = NULL;
		
		// Open Base Directory.
		// (The '@' before the call is to suppress the error if the directory is not found.)
		$dhandle = @opendir($basedir);
		if ($dhandle != false) {
		
			// Loop Through Files in Directory
			$dirname = readdir($dhandle);
			while ($dirname != false) {
				if ( $dirname != "." && $dirname != ".." ) { 
				
					// Year Folder
					if ( is_dir( $basedir . $dirname ) ) {
						
						// Open Year Folder
						$subdhandle = opendir( $basedir . $dirname . '/' );
						if ($subdhandle != false) {
												
							// Loop Through Files in Directory
							$subdirname = readdir($subdhandle);
							while ( $subdirname != false ) {
								if ( $subdirname != "." && $subdirname != ".." ) { 
								
									// Month Folder
									if ( is_dir( $basedir.$dirname.'/'.$subdirname ) ) {
										
										if ( intval( $dirname ) >= intval( $latest_y ) ) {
											if ( intval( $subdirname ) >= intval( $latest_m ) ) {
												$latest_y = $dirname;
												$latest_m = $subdirname;
												
												$entries = sb_folder_listing( $basedir.$dirname.'/'.$subdirname.'/', array( '.txt', '.gz' ) );
												if ( count( $entries ) > 0 ) {
													$latest_d = substr( $entries[ count( $entries ) - 1 ], 9, 2 );
												}
											}
										}
									}
									
								}
								$subdirname = readdir($subdhandle);
							}
							
						}
						closedir($subdhandle);
					}
					
				}
				$dirname = readdir($dhandle);
			}
			
			closedir($dhandle);
		}
	
		if ( isset($latest_y) ) {
			$GLOBALS['year'] = $latest_y;
			$GLOBALS['month'] = $latest_m;
			$GLOBALS['day'] = $latest_d;
		}
	}
	
	function read_menus ( $m, $y, $d ) {
		// Create the right-hand navigation menu. Return HTML
		//
		global $lang_string, $default_view;
		
		$str = NULL;
		clearstatcache();
		
		$basedir = 'content/';
		
		$months = $lang_string['sb_months'];
		
		// Open Base Directory
		// (The '@' before the call is to suppress the error if the directory is not found.)
		$dhandle = @opendir($basedir);
		if ($dhandle != false) {
			// Loop Through Files in Directory
			$dirname = readdir($dhandle);
			while ($dirname != false) {
				if ( $dirname != '.' && $dirname != '..' && $dirname != 'static') { 
				
					// Year Folder
					if (is_dir( $basedir . $dirname )) {
						
						// Store Year Text
						$str_year = '20' . $dirname . '<br />';
						
						// Open Year Folder
						$subdhandle = opendir($basedir . $dirname . '/' );
						if ($subdhandle != false) {
												
							// Loop Through Files in Directory
							$subdirname = readdir($subdhandle);
							$str_month = NULL;
							
							while ($subdirname != false) {
								if ( $subdirname != "." && $subdirname != ".." ) { 
								
									// Month Folder
									if ( is_dir( $basedir.$dirname.'/'.$subdirname ) ) {
										$entries = sb_folder_listing( $basedir.$dirname.'/'.$subdirname.'/', array( '.txt', '.gz' ) );
										
										if ( intval( $subdirname ) == $m && intval( $dirname ) == $y ) {
											// Currently Selected Month
											
											// Loop Through Days
											for ( $i = 0; $i < count( $entries ); $i++ ) {
												$temp_day = substr( $entries[$i], 9, 2 );
												$temp_month = substr( $entries[$i], 7, 2 );
												$temp_year = substr( $entries[$i], 5, 2 );
												$stamp = strtotime( $temp_month . '/' . $temp_day . '/' . $temp_year );
												$temp_date = format_date_menu( $stamp );
												
												// Count the number of entries on this day
												$temp_count = 1;
												for ( $j = $i + 1; $j < count( $entries ); $j++ ) {
													if ( $temp_day == substr( $entries[$j], 9, 2 ) &&
														 $temp_month == substr( $entries[$j], 7, 2 ) &&
														 $temp_year == substr( $entries[$j], 5, 2 ) ) {
														$temp_count++;
													} else {
														break;
													}
												}
												$i = $j - 1;
													
												if ( $temp_day == $d && $default_view == false ) {
													// Currently Selected Day
													$str_month = '&nbsp;&nbsp;&nbsp;' . $temp_date . ' ( ' . $temp_count . ' )' . '<br />' . $str_month;
												} else {
													if ( $default_view != 2) {
														// Non-Selected Days
														$str_month = '<a href="index.php?m='.$subdirname.'&y='.$dirname.'&d='.$temp_day.'">&nbsp;&nbsp;&nbsp;' . $temp_date  . ' ( ' . $temp_count . ' )' . '</a><br />' . $str_month;
													}
												}
											}
											
											$str_month = '<a href="index.php?m='.$subdirname.'&y='.$dirname.'&default_view=1">' . $months[ intval($subdirname, 'integer') - 1 ].' ( '.count( $entries ).' )</a><br />' . $str_month;
										} else {
											// Non-Selected Months
											if ( count( $entries ) > 0 ) {
												$latest_d = substr( $entries[ count( $entries ) - 1 ], 9, 2 );
												$str_month = '<a href="index.php?m='.$subdirname.'&y='.$dirname.'&d='.$latest_d.'&default_view=1">'.$months[ intval($subdirname, 'integer') - 1 ].' ( '.count( $entries ).' )</a><br />' . $str_month;
											} else {
												$str_month = '<a href="index.php?m='.$subdirname.'&y='.$dirname.'&default_view=1">'.$months[ intval($subdirname, 'integer') - 1 ].' ( '.count( $entries ).' )</a><br />' . $str_month;
											}
										}
									}
								}
								
								$subdirname = readdir($subdhandle);
							}
							
							$str_year = $str_year.$str_month;
							$str = $str_year . $str;
							
						}
						closedir($subdhandle);
					}
				}
				$dirname = readdir($dhandle);
			}
			closedir($dhandle);
		}
		
		return ( $str );
	}
	
	function read_links ( $logged_in ) {
		// Create the right-hand link menu. Return HTML
		//
	
		global $lang_string;
		
		// Read links file.
		$filename = 'config/links.txt';
		$result = sb_read_file( $filename );
		
		// Append new links.
		$str = NULL;
		if ( $result ) {
			$array = explode('|', $result);
			for ( $i = 0; $i < count( $array ); $i = $i + 2 ) {
				if ( $array[$i+1] == '' ) {
					$str = $str . '<br />' . $array[$i] . '<br />';
				} else {
					$str = $str . '<a href="' . $array[$i+1] . '">' . $array[$i] . '</a><br />';
				}
			}
		}
		
		if ( $logged_in === true ) {
			$str = $str . '<a href="add_link.php">[ ' . $lang_string['sb_add_link_btn']  . ' ]</a><br />';
		}
		
		return ( $str );
	}
	
	function write_link ( $link_name, $link_url, $link_id ) {
		// Save new link. Update links file
		//
		
		// write_link( clean_post_text( $blog_subject ), 'static.php?page='.$entryFile, $i-1 );
		
		// Clean up link name and make safe for HTML and text database storage.
		global $lang_string;
		$link_name = str_replace( '|', ':', $link_name );
		$link_name = htmlspecialchars( $link_name, ENT_QUOTES, $lang_string['php_charset'] );
		
		// Clean up link url and make safe text database storage.
		$link_url = str_replace( '|', ':', $link_url );
		
		// Read old links file.
		$filename = 'config/links.txt';
		$result = sb_read_file( $filename );
	
		// Append new links.
		if ( $result ) {
			$array = explode('|', $result);
			
			if ( $link_id !== "" ) {
				array_splice( $array, $link_id, 2 );
				array_splice( $array, $link_id, 0, array( $link_name, $link_url ) );
			} else {
				array_push( $array, $link_name );
				array_push( $array, $link_url );
			}
		} else {
			$array = array( $link_name, $link_url );
		}
		
		// Save links to file.
		$str = implode('|', $array);
		$result = sb_write_file( $filename, $str );
		
		if ( $result ) {
			return ( true );
		} else {
			// Error:
			// Probably couldn't create file...
			return ( $filename );
		}
	}
	
	function modify_link ( $action, $link_id ) {
		// Modify links.
		// Move links up or down, edit or delete.
		
		// Read links file.
		$filename = 'config/links.txt';
		$result = sb_read_file( $filename );
		
		// Append new links.
		if ( $result ) {
			$array = explode('|', $result);
			
			if ( $action === "up" ) {
				if ( count( $array ) > 2 && $link_id != 0 ) {
					$pop_array = array_splice( $array, $link_id, 2 );
					array_splice( $array, $link_id-2, 0, $pop_array );
				}
			}
			if ( $action === "down" ) {
				if ( count( $array ) > 2 && $link_id < ( count( $array ) - 3 ) ) {
					$pop_array = array_splice( $array, $link_id, 2 );
					array_splice( $array, $link_id+2, 0, $pop_array );
				}
			}
			if ( $action === "delete" ) {
				if ( $link_id <= ( count( $array ) - 1 ) ) {
					array_splice( $array, $link_id, 2 );
				}
			}
			if ( $action === "delete_static" ) {
				for ( $i = 0; $i < count( $array ); $i++ ) {
					if ( $link_id == $array[$i] ) {
						array_splice( $array, $i-1, 2 );
						break;
					}
				}
			}
		}
		
		// Save links to file.
		$str = implode('|', $array);
		$result = sb_write_file( $filename, $str );
		
		if ( $result ) {
			return ( true );
		} else {
			// Error:
			// Probably couldn't create file...
			return ( $filename );
		}
	}
	
	// -----------------------
	// 'Most Recent' Functions
	// -----------------------
	
	function add_most_recent ( $comment_id, $y, $m, $blog_entry_id ) {
		// Add an item to the 'Last Updated' List
		//
		
		// Read links file.
		$filename = 'config/last_updated.txt';
		$result = sb_read_file( $filename );
		
		// Append new links.
		if ( $result ) {
			$array = explode('|', $result);
			array_push( $array, $blog_entry_id, $m, $y, $comment_id );
		} else {
			$array = array( $blog_entry_id, $m, $y, $comment_id );
		}
		
		$max_comments = 5;
		if ( count( $array ) > ( ( $max_comments * 4 ) - 1 ) ) {
			// $array = array_reverse( $array );
			$array = array_slice( $array, $max_comments * -4, $max_comments * 4);
			// $array = array_reverse( $array );
		}
		
		// Save links to file.
		$str = implode( '|', $array );
		sb_write_file( $filename, $str );
	}
	
	function delete_most_recent ( $item_filename ) {
		// Delete an item to the 'Last Updated' List
		//
		
		// Read links file.
		$filename = 'config/last_updated.txt';
		$result = sb_read_file( $filename );

		$blog_entry_id = sb_strip_extension( strrchr( $item_filename, '/') );
		
		// Append new links.
		$str = NULL;
		$update_file = false;
		if ( $result ) {
			$array = explode('|', $result);
			$array = array_reverse( $array );
			for ( $i = 0; $i < count( $array ); $i = $i + 4 ) {
				if ( $blog_entry_id == $array[$i] ) {
					array_splice( $array, $i, 4 );
					$update_file = true;
					break;
				}
			}
		}
		
		// Save links to file.
		if ( $update_file ) {
			$array = array_reverse( $array );
			$str = implode('|', $array);
			sb_write_file( $filename, $str );
		}
	}
	
	function get_most_recent () {
		// Read last updated items from disk, return HTML
		//
		global $lang_string, $user_colors;
		
		// Read links file.
		$filename = 'config/last_updated.txt';
		$result = sb_read_file( $filename );
		
		// Append new links.
		$str_comments = NULL;
		if ( $result ) {
			$array = explode('|', $result);
			$array = array_reverse( $array );
			for ( $i = 0; $i < count( $array ); $i = $i + 4 ) {
				$comment_id = $array[$i+0];
				$y = $array[$i+1];
				$m = $array[$i+2];
				$blog_entry_id = $array[$i+3];
				
				$comment_file = 'content/'.$y.'/'.$m.'/'. sb_strip_extension( $blog_entry_id ).'/'.$comment_id;
				if ( file_exists( $comment_file . '.txt' ) ) {
					$comment_file = $comment_file . '.txt';
				} elseif ( file_exists( $comment_file . '.txt.gz' ) ) {
					$comment_file = $comment_file . '.txt.gz';
				}
			
				$comment_contents = sb_read_file( $comment_file );
				if ( $comment_contents ) {
					list( $comment_name, $comment_date, $comment_text ) = explode('|', $comment_contents );
					
					$comment_text = blog_to_html( $comment_text, false, true );
					
					if ( strlen( $comment_text ) > 40 ) {
						$comment_text = substr( $comment_text, 0, 40 );
						$comment_text = substr( $comment_text, 0, strrpos( $comment_text, ' ' ) ) . '...';
					}
					if ( strlen( $comment_name ) > 40 ) {
						$comment_name = substr( $comment_name, 0, 40 );
						$comment_name = substr( $comment_name, 0, strrpos( $comment_name, ' ' ) ) . '...';
					}
					
					global $blog_config;
					if ( $blog_config[ 'blog_comments_popup' ] == 1 ) {
						$str_comments = $str_comments . '<a href="javascript:openpopup(\'comments.php?y='.$y.'&m='.$m.'&entry='.$blog_entry_id.'\',500,500,true)">'.$comment_name.'</a><br />';
					} else {
						$str_comments = $str_comments . '<a href="comments.php?y='.$y.'&m='.$m.'&entry='.$blog_entry_id.'">'.$comment_name.'</a><br />';
					}
					
					$str_comments = $str_comments . format_date( $comment_date ) . '<br />';
					$str_comments = $str_comments . $comment_text . '<p />';
				}
			}
		}
		
		return ( $str_comments );
	}
	
	function compress_all_files () {
		// This function compresses or decompressed all
		// data files. I would recommend backing all your
		// blog entries up before you run this. :)
		//
		// 1.2MB - 135,457 bytes
		global $blog_config;
		
		clearstatcache();
		
		$basedir = 'content/';
		
		// YEAR directories
		$counter = 0;
		$dir = $basedir;
		if ( is_dir( $dir ) ) {
			if ( $year_dir_handle = @opendir( $dir ) ) {
				while ( ( $year_dir = readdir( $year_dir_handle ) ) !== false ) {
					if ( is_dir( $dir.$year_dir . '/' ) ) {
						if ( $year_dir != '.' && $year_dir != '..' ) {
							if ( $year_dir != 'static' ) {
						
								// MONTH directories
								
								if ( $month_dir_handle = @opendir( $dir.$year_dir . '/' ) ) {
									while ( ( $month_dir = readdir( $month_dir_handle ) ) !== false ) {
										if ( is_dir( $dir.$year_dir.'/'.$month_dir . '/' ) ) {
											if ( $month_dir != '.' && $month_dir != '..' ) {
											
												// ENTRIES
								
												if ( $entry_dir_handle = @opendir( $dir.$year_dir.'/'.$month_dir . '/' ) ) {
													while ( ( $entry_filename = readdir( $entry_dir_handle ) ) !== false ) {
														if ( is_file( $dir.$year_dir.'/'.$month_dir.'/'.$entry_filename ) ) {
															 
															// Store Filename
															$ext = strtolower( strrchr( $entry_filename, '.' ) );
															if ( $blog_config[ 'blog_enable_gzip_txt' ] ) {
																if ( $ext == '.txt' ) {
																	$str = sb_read_file( $dir.$year_dir.'/'.$month_dir.'/'.$entry_filename );
																	if ( $str ) {
																		$ok = sb_write_file( $dir.$year_dir.'/'.$month_dir.'/'.sb_strip_extension( $entry_filename ) . '.txt.gz' , $str );
																		if ( $ok ) {
																			sb_delete_file( $dir.$year_dir.'/'.$month_dir.'/'.$entry_filename );
																			$counter++;
																		}
																	}
																}
															} else {
																if ( $ext == '.gz' ) {
																	$str = sb_read_file( $dir.$year_dir.'/'.$month_dir.'/'.$entry_filename );
																	if ( $str ) {
																		$ok = sb_write_file( $dir.$year_dir.'/'.$month_dir.'/'.sb_strip_extension( $entry_filename ) . '.txt' , $str );
																		if ( $ok ) {
																			sb_delete_file( $dir.$year_dir.'/'.$month_dir.'/'.$entry_filename );
																			$counter++;
																		}
																	}
																}
															}
														} else {
															
															// COMMENTS
															
															$comments_dir = $entry_filename;
															if ( is_dir( $dir.$year_dir.'/'.$month_dir.'/'.$comments_dir.'/' ) ) {
																if ( $comments_dir != '.' && $comments_dir != '..' ) {
																	if ( $comments_dir_handle = @opendir( $dir.$year_dir.'/'.$month_dir.'/'.$comments_dir.'/' ) ) {
																	
																		while ( ( $comment_filename = readdir( $comments_dir_handle ) ) !== false ) {
																			if ( is_file( $dir.$year_dir.'/'.$month_dir.'/'.$comments_dir.'/'.$comment_filename ) && $comment_filename != 'view_counter.txt' ) {
																				 
																				// Store Filename
																				$ext = strtolower( strrchr( $comment_filename, '.' ) );
																				if ( $blog_config[ 'blog_enable_gzip_txt' ] ) {
																					if ( $ext == '.txt' ) {
																						$str = sb_read_file( $dir.$year_dir.'/'.$month_dir.'/'.$comments_dir.'/'.$comment_filename );
																						if ( $str ) {
																							$ok = sb_write_file( $dir.$year_dir.'/'.$month_dir.'/'.$comments_dir.'/'.sb_strip_extension( $comment_filename ) . '.txt.gz' , $str );
																							if ( $ok ) {
																								sb_delete_file( $dir.$year_dir.'/'.$month_dir.'/'.$comments_dir.'/'.$comment_filename );
																								$counter++;
																							}
																						}
																					}
																				} else {
																					if ( $ext == '.gz' ) {
																						$str = sb_read_file( $dir.$year_dir.'/'.$month_dir.'/'.$comments_dir.'/'.$comment_filename );
																						if ( $str ) {
																							$ok = sb_write_file( $dir.$year_dir.'/'.$month_dir.'/'.$comments_dir.'/'.sb_strip_extension( $comment_filename ) . '.txt' , $str );
																							if ( $ok ) {
																								sb_delete_file( $dir.$year_dir.'/'.$month_dir.'/'.$comments_dir.'/'.$comment_filename );
																								$counter++;
																							}
																						}
																					}
																				}
																			}
																		}
																	
																	}
																}
															}
															
															// END of COMMENTS
															
														}
													}
												}
												
												// END of ENTRIES
												
											}
										}
									}
								}
								
								// END of MONTH directories
							} else {
							
								// STATIC ENTRIES ( $year_dir == 'static' )
								
								if ( $static_dir_handle = @opendir( $dir.$year_dir . '/' ) ) {
									while ( ( $entry_filename = readdir( $static_dir_handle ) ) !== false ) {
										if ( is_file( $dir.$year_dir . '/' .$entry_filename ) ) {
											 
											// Store Filename
											$ext = strtolower( strrchr( $entry_filename, '.' ) );
											if ( $blog_config[ 'blog_enable_gzip_txt' ] ) {
												if ( $ext == '.txt' ) {
													$str = sb_read_file( $dir.$year_dir.'/'.$entry_filename );
													if ( $str ) {
														$ok = sb_write_file( $dir.$year_dir.'/'.sb_strip_extension( $entry_filename ) . '.txt.gz' , $str );
														if ( $ok ) {
															sb_delete_file( $dir.$year_dir.'/'.$entry_filename );
															$counter++;
														}
													}
												}
											} else {
												if ( $ext == '.gz' ) {
													$str = sb_read_file( $dir.$year_dir.'/'.$entry_filename );
													if ( $str ) {
														$ok = sb_write_file( $dir.$year_dir.'/'.sb_strip_extension( $entry_filename ) . '.txt' , $str );
														if ( $ok ) {
															sb_delete_file( $dir.$year_dir.'/'.$entry_filename );
															$counter++;
														}
													}
												}
											}
											
										}
									}
								}
								
								// END of STATIC ENTRIES
			
							}
						}
					}
				}
			}
		}
		return( $counter );
	}
	
	function sb_mail ($from, $to, $subject, $body, $text=true, $priority=3)
	{
		// Contributed by: Javier Gutierrez, guti <at> ya <dot> com
		//
		global $sb_info;
	
	
		$headers="";
		if ($text)
		{
			$headers=$headers . "Content-Type: text/plain \r\n";
		}
		else
		{
			$headers=$headers . "MIME-Version: 1.0 \r\n";
			$headers=$headers . "Content-type: text/html; charset=iso-8859-1 \r\n";
		}
		$headers=$headers . "From: " . $from . " \r\n";
		$headers=$headers . "Reply-To: " . $from . " \r\n";
		$headers=$headers . "Return-Path: " . $from . " \r\n";
		$headers=$headers . "Date: " . date("r") . " \r\n";
		$headers=$headers . "X-Priority: " . $priority . " \r\n";
		$headers=$headers . "X-Mailer: SPHPBLOG/".$sb_info[ 'version' ]." \r\n";
		ini_set("sendmail_from", $from);
		$res=mail($to, stripslashes($subject), stripslashes($body), $headers);
		ini_restore("sendmail_from");
		return($res);
	}
	
	// Activate PHP's GZ compression output, if not currently activated.
	// Must be called before any header output.
	//
	function sb_gzoutput ()
	{
		// Contributed by: Javier Gutierrez, guti <at> ya <dot> com
		//
	  	if ( ( ini_get( "zlib.output_compression" ) != "0" ) && ( ini_get("zlib.output_compression" ) != "On" ) && ( extension_loaded('zlib') ) )
		{
			ini_set( "zlib.output_compression_level", 9);
			ob_start( "ob_gzhandler" );
			ini_restore( "zlib.output_compression_level" );
		}
	}
	
	// Send a update ping to a ping server in url.
	//
	function sb_ping ( $url )
	{
		// Contributed by: Javier Gutierrez, guti <at> ya <dot> com
		//
		if ( $url == NULL || $url == '' ) {
			return( false );
		}
		
		global $blog_config, $sb_info;

		$result="";
		$user_agent = "SPHPBLOG ping script/" . $sb_info[ 'version' ];
		$url = parse_url($url);

		$xml="<?xml version=\"1.0\"?>\n";
		$xml=$xml . "<methodCall>\n";
		$xml=$xml . "\t<methodName>weblogUpdates.ping</methodName>\n";
		$xml=$xml . "\t<params>\n";
		$xml=$xml . "\t\t<param><value>" . htmlspecialchars( strip_tags( $blog_config[ 'blog_title' ] ) ) . "</value></param>\n";
		$xml=$xml . "\t\t<param><value>http://" . $_SERVER['HTTP_HOST'] . dirname( $_SERVER['PHP_SELF'] ) . "</value></param>\n";
		$xml=$xml . "\t</params>\n";
		$xml=$xml . "</methodCall>\n";
		$socket=fsockopen( $url['host'], 80, $errno, $errstr, 30);
		if ( $socket ) { 
			fwrite( $socket, "POST " . $url['scheme'] . '://' . $url['host'] . $url['path'] . " HTTP/1.0\nHost: http://" . $_SERVER['HTTP_HOST'] . dirname( $_SERVER['PHP_SELF'] ) . "\r\nUser-Agent:" . $user_agent . " 0.1\nContent-Type: text/xml\nContent-Length: " . strlen ( $xml ) . "\n\n" . $xml . "\n" );
			$result=fread( $socket, 8192 );
			fclose ( $socket );
			return( strpos( $result, "fault" )===false );
		}
		else {
			return(false);
		}
	}
?>